/* ============================================================================
 * StartFTPServer.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.server;

import java.io.File;
import java.io.PrintWriter;
import nl.jhhcvandermeijs.ftp.server.TFTPProtocolSettingsAndState.FTPProtocolData;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;


/**
 * A class that instantiates a TFTPServer
 */
public class StartFTPServer {

    public static void createEmptySettingsFile() {
        FTPProtocolData data = new FTPProtocolData();

        String logFileDir = Utils.validateDirectory(TFTPServer.getAppdir() + "ftpserverlog"); if (!(new File(logFileDir).exists())) {(new File(logFileDir)).mkdir();}
        data.setMaxClients(256);
        data.setServerPort(2100);
        data.setBindIPAddress("0.0.0.0");
        data.setExternalIPOfThisServer(Utils.getNetworkIp());
        data.setTrackChanges(false);

        //ADD LOG SETTINGS
        data.setLogDirectory(logFileDir);
        data.setMainLogFileName(logFileDir + "ftpserverlog.txt");
        data.setLogEvents(true);
        data.setLogAppend(false);

        //ADD TLS SETTINGS
        data.setAllowTLS(false);
        data.setImplicitTLS(false);
        data.setEnforceExplicitTLSWhenNotImplicit(false);
        data.setEnforcePROTPWhenExplicit(false);
        data.setRequireClientAuthentication(false);
        data.setPEMEncodedServerCertAndKeyFileName(TFTPServer.getAppdir() + "X509CertAndKey" + File.separator + "server.pem");
        data.setKeyPassphrase("aPassphrase");
        data.setTrustedCACertsDirectory(TFTPServer.getAppdir() + "TrustedCAs");

        //ADD GROUP ANONYMOUS
        FTPProtocolData.Group aGroup = new FTPProtocolData.Group();
        aGroup.setGroupName(TFTPProtocol.ANONYMOUS);
        aGroup.setOpeningDirectory((Utils.validateDirectory(TFTPServer.getAppdir() + TFTPProtocol.ANONYMOUS)));
        data.getGroupsList().clear();
        data.getGroupsList().add(aGroup);

            //ADD FOLDER
            FTPProtocolData.Folder aFolder = new FTPProtocolData.Folder();
            aFolder.setFolderName(Utils.validateDirectory(TFTPServer.getAppdir() + TFTPProtocol.ANONYMOUS));

            aFolder.setIsAllowedToSeeFiles(true);
            aFolder.setIsAllowedToReadFiles(true);
            aFolder.setIsAllowedToRenameFiles(false);
            aFolder.setIsAllowedToAppendToFiles(false);
            aFolder.setIsAllowedToCreateNewFiles(false);
            aFolder.setIsAllowedToOverwriteFiles(false);
            aFolder.setIsAllowedToDeleteFiles(false);

            aFolder.setIsAllowedToSeeSubDirectories(true);
            aFolder.setIsAllowedToTraverseSubDirectories(true);
            aFolder.setIsAllowedToMakeSubDirectories(false);
            aFolder.setIsAllowedToRenameSubDirectories(false);
            aFolder.setIsAllowedToRemoveSubDirectories(false);

            aFolder.setIsAllowedToSeeHiddenFiles(false);
            aFolder.setIsAllowedToReadHiddenFiles(false);
            aFolder.setIsAllowedToRenameHiddenFiles(false);
            aFolder.setIsAllowedToAppendToHiddenFiles(false);
            aFolder.setIsAllowedToCreateNewHiddenFiles(false);
            aFolder.setIsAllowedToOverwriteHiddenFiles(false);
            aFolder.setIsAllowedToDeleteHiddenFiles(false);

            aFolder.setIsAllowedToSeeHiddenSubDirectories(false);
            aFolder.setIsAllowedToTraverseHiddenSubDirectories(false);
            aFolder.setIsAllowedToMakeHiddenSubDirectories(false);
            aFolder.setIsAllowedToRenameHiddenSubDirectories(false);
            aFolder.setIsAllowedToRemoveHiddenSubDirectories(false);

            aGroup.getFoldersList().clear();
            aGroup.getFoldersList().add(aFolder);

        //ADD USER ANONYMOUS
        FTPProtocolData.User aUser = new FTPProtocolData.User();
        aUser.setUserName(TFTPProtocol.ANONYMOUS);
        aUser.setPassphrase(TFTPProtocol.ANONYMOUS);
        aUser.setRequirePassphraseToBeCorrect(false); //normally should be TRUE!!!
        aUser.setAccountIsActive(true);
        aUser.setGroupName(TFTPProtocol.ANONYMOUS);
        data.getUsersList().clear();
        data.getUsersList().add(aUser);

        //SAVE XML
        data.saveToXML(TFTPServer.getConfigurationFileName());
    }

    public static void writeSampleCerts() {
        PrintWriter pem = null;
        try {
            String anonymous = Utils.validateDirectory(TFTPServer.getAppdir() + "anonymous");
            if (!(new File(anonymous)).exists()) {(new File(anonymous)).mkdirs();}

            String serverX509CertAndKeyDir = Utils.validateDirectory(TFTPServer.getAppdir() + "X509CertAndKey");
            if (!((new File(serverX509CertAndKeyDir)).exists())) {(new File(serverX509CertAndKeyDir)).mkdir();}
            if (!((new File(serverX509CertAndKeyDir + "server.pem")).exists())) {
                    //create sample server.pem
                    pem = new PrintWriter(serverX509CertAndKeyDir + "server.pem");
                    pem.write("-----BEGIN CERTIFICATE-----\r\n");
                    pem.write("MIIFODCCAyCgAwIBAgIGASEMID2QMA0GCSqGSIb3DQEBDQUAMEExPzA9BgNVBAMM\r\n");
                    pem.write("NkZUUFNFUlZFUkNBX1dBUk5JTkdfX1BMRUFTRV9HRU5FUkFURV9ZT1VSX09XTl9f\r\n");
                    pem.write("V0FSTklORzAeFw0wOTA1MDMwMTM0MjlaFw0xNDEwMTQwMDQyNTlaMFQxUjBQBgNV\r\n");
                    pem.write("BAMMSUZUUFNFUlZFUl9TRVJWRVJYNTA5Q0VSVEFOREtFWV9XQVJOSU5HX19QTEVB\r\n");
                    pem.write("U0VfR0VORVJBVEVfWU9VUl9PV05fX1dBUk5JTkcwggIiMA0GCSqGSIb3DQEBAQUA\r\n");
                    pem.write("A4ICDwAwggIKAoICAQCOcobhhMmDlYOjX5/Q1gPh596YmYNcdHhszco7AFbuMQzl\r\n");
                    pem.write("upSpfetd5YsfPJ9u05cMkT0e828C4rdw4g7inKPsYeT6dTOiz4AeechnOYdljpuw\r\n");
                    pem.write("N+tk1dmX4E6j5NIG8GmmRbqIBR9exl/aM4yWBkV9bjxTwtsqBxcphmqtr2JYJUvQ\r\n");
                    pem.write("r1urLqJWhBNv9YnC5VrcPzljM3Mzpd61HazKetKRf0mbElRlxooam+tP5bYiF1YZ\r\n");
                    pem.write("XkMJsTRvc+SMOFaWdoG2upxmlWSDdIcsGtUTIDzIK58MeJp7xOMDFIHNXriCcKA7\r\n");
                    pem.write("O7+WfiM39ZeD+0skC740KBWc93xAvFhwPZDV7BjzKhthq1DJjd06X5N20uiPr3pq\r\n");
                    pem.write("zZZcUdaMya2gMy3Jsa1ssSVEKK73vC/ZUlT/ymVK0QoyN4iESw4Xl7KJALQl7nHp\r\n");
                    pem.write("tVcdVGF/icqp5rylsvdlo359Ftthr3yO9gtMvVlMryiBGwyJODeYsmZaiw9jnWiq\r\n");
                    pem.write("FEE2tVM9qILkGDruv6ok3sdjeMA9vc9MfQ1Foal0rrxTgOijxNsKRkHwilFFyKrR\r\n");
                    pem.write("O45R3dDR+VYVKvpS+y11x6Cx/vcYbvAYGKwU7dDCETMJzcOKQB9kCayUGETTmXvJ\r\n");
                    pem.write("9l/nuRJc8ZDinkvUydqcQNr4t/bXAEuA7hUM6AU3cu6a28nPv7qqecsnmfbMeQID\r\n");
                    pem.write("AQABoyMwITAPBgNVHRMBAf8EBTADAQH/MA4GA1UdDwEB/wQEAwIB/jANBgkqhkiG\r\n");
                    pem.write("9w0BAQ0FAAOCAgEAMt7RmRa+gG7a8zHyNNztqmkFn35aoQ5tzLN3oSP8ZCX1gP5q\r\n");
                    pem.write("i7bj3qw2mTXhDYwQS/eW2+Jtw6T5lJBREGIzpyqAMysuGGP8ICqPCRkuhABYx/rR\r\n");
                    pem.write("LWJa6Nn8e3s9TDCPD0Ec0wzOZp8BJzb0PqZnz/0ljq75a7IHU11ZIIgA1K+uHNbx\r\n");
                    pem.write("k3xxogxEtl+qXJ3X8ayRZOSEzAUYz5WwBj1gUdMn0xsmsnh/w9FoKk7an+t2wz7x\r\n");
                    pem.write("YtXaPvLYkSDcWJuf4fnW4YXXxS6PqOqEdTH2X3IVFh3kPGypqFMUcGhbAK7thfsW\r\n");
                    pem.write("BY8GHpI0Fo6XekagYyWAzKlBUUa4sQ4ShOMsydIMl1JkDLnGolo+g16Oflq2YI1C\r\n");
                    pem.write("c4O0uWOlcMMyM91zpZXomV8a+n87SHvOzSH42KQLp4o9ZISbq5oOUm/guMBpltVF\r\n");
                    pem.write("P1tvKTGAQBBU+hOxPM5Yvbq5gPl6XGy5PtP954vea7bGZNUzNc5xo5KueEV+KeoV\r\n");
                    pem.write("qg6ODevppfyUdpP3kcEVHwlL5Xvinopc4tLRRVjhFfrKPuPaSnA6I/Fj3Gpfmz60\r\n");
                    pem.write("56YZC0vrqEDrm6EPDcST2POBLQmiSwHJJK/BprGw8erN+UirUGEfZSAMoUtK5UCT\r\n");
                    pem.write("S0Tv3aVg5rWuF6oTx10178BZ8Gf2CL3mHBoJVJ1LnQ2v9k1HWYJ/JeV+THM=\r\n");
                    pem.write("-----END CERTIFICATE-----\r\n");
                    pem.write("-----BEGIN RSA PRIVATE KEY-----\r\n");
                    pem.write("Proc-Type: 4,ENCRYPTED\r\n");
                    pem.write("DEK-Info: AES-256-CFB,34c84f3ca0b7b263816210a8854f5134\r\n");
                    pem.write("\r\n");
                    pem.write("utdORBP8lKifayi4+DgUyIwh6DPREjUXTyKE3IrqwYEPfj1UOrTVQgU2dGdFokP+\r\n");
                    pem.write("iiEJHFx/7+1QB4jCi2s/XvwGIjvW+rhNcLBh035JO5gcn/v8Y7YbV2tv+J5REk8D\r\n");
                    pem.write("hCrKg9GdJajuZ0Bdecu2XAztPDFkDlSCclf4cnPaVtn8Y/QyoCebLClQp/RMD2pl\r\n");
                    pem.write("UGw709iZg7TJKsRbBRBsdOT2nwZ1dzICRADI3gzsqmRaqOAgN5MHi1Twl8SC5U0a\r\n");
                    pem.write("LN4fZCoO1qy67kKa970JRWI59JrYIW4Z2baJ1S0xDAv9a/6vMwVJKqlHiWMyg9GQ\r\n");
                    pem.write("wb8GjXiD4oGbVvM4z8YXH/k9CPujJ7vKg8WflL8eVy+Zzdqz87UpMr0Tj7BKKD4a\r\n");
                    pem.write("wzAT4DgZqXgz3gJoskurL/iJG9BZqE/PANDAOgbXdkprrDuja1TZ0NeKdkgbsHeX\r\n");
                    pem.write("cqMRf93DarsLeAAuPfE6nqI1ke684a8FU+/GuwyWJuLG3s7wBYM5PTt0nheowH8S\r\n");
                    pem.write("ihmpNhFPxMkx5LuINSleU2bXQfCPKPOBjDWJQYDOUqgvu4D1he8iNsF7l39uEDmf\r\n");
                    pem.write("IwTW3IWTGvPotOrGWIDKBqUX0a3cElnTfyX7AVTyL2HX7xJd0gWn7E4rhurRTi/K\r\n");
                    pem.write("l/gCXIggGDM2wylyoWCmkzk8yPqU+JbUo1HTnsWalP3MSX8Hrbpa1TEM8zGacimn\r\n");
                    pem.write("lr2lZi9ZAEbwvchrfFe40/5Mz3ShyfdiMpabG87tyOP3NEP0bypqFXlGhzSr9Pin\r\n");
                    pem.write("kezvKTEDgSpEv8Q9zo5ae2K/1d05tfwBiqsCQqufXiGsdCFR9g/8sizpnuH9c1bK\r\n");
                    pem.write("ZotQsJAkpJ9KwuEKna5A7SkycJiXum++OwdoYeFS1KDVAXHMA/8/kLa4YSxMkXM+\r\n");
                    pem.write("NIVKuSIsDWiIH54OM8g9qwxeTH6dL5WRR+EqAepK7n7eZtIolCA8KyT5OFPupDNI\r\n");
                    pem.write("LeP/4lL2LHyIlAf0lyBg3oGOKOWKc60xl7K3X97Vy4YeyUxLzZXZ+vXgviF/RS+2\r\n");
                    pem.write("9fYbYhZQBjFQMaOL1XM0EVVG4EMxKkNueLrJisYgnCbuwgEJmyJLr8TuN89kPKeO\r\n");
                    pem.write("UHsljf6SKATlfS85SaW5F0RIovrhQdt7wRrVx5kHzLNZRDJJ4jk3shwsdN5P+Spd\r\n");
                    pem.write("L3aF4pS7U6WlK9OeawYDSHHJXaqTITM4Md+jBRiLoUZZI7wRnUuViDHv9i/9s9Rr\r\n");
                    pem.write("a0TscwT6YLIlBG1uEVNRnHzIFHew5QKfkuv7zmUllo19yVmYjk96JCYsokr7aHpD\r\n");
                    pem.write("518YwWavkGLb8SCAOdpvdX1JyflaIoLafwWFf8LjvJIjW4dD45LTU17tEE4v73Fe\r\n");
                    pem.write("rYZ3WNOdrjuQwLWU2rdjGjkDZgq0ATM0Nw2/UNaEpPFb0fxdH92FDG3synzO6Ycy\r\n");
                    pem.write("mo4CXE6OqaA716TZGhE3XGjp7zz98HcsnwEfzjC1C5T++gRhnUun8PU80MqzCvBF\r\n");
                    pem.write("qd3Tc/wSx1ICaQutvCEziuCVxQeN4uwl9PGE/2XyZnKyjadt6mPHQofYCRAbiFVv\r\n");
                    pem.write("UffZY4cTL5vO/Dxm5CLT+3rqRLRgDggDiKwzybGh7uwtx0HdltjhZ07wcrKin90e\r\n");
                    pem.write("IIdrVxUwcK93PvZ+F7ZL73bOIG2Lv55E9b96FE/s+gpflVWVgbnHEDaKHVvmT2IO\r\n");
                    pem.write("SwF/8SCnqCCaLEtxpYrQFTzKQT+lokZmAb/EdWHMPWo9XiM1I06YUZH0/dVNshbe\r\n");
                    pem.write("1OP/iA6rEwD+4eDPwpwBGhnpdczWYqsHundgW/xkWmvzQ0owSgQUF+BgD3HkY8ga\r\n");
                    pem.write("CkVv1cHUj5lcLkwjkF1PjF5a7NyQox/nHTKD9V0LuxO/PHYzVhpniT3zjv3MwQvb\r\n");
                    pem.write("9uXSl1I8msiG195ZF6hPDTy4uXWtqaEIs6ekcYftezzWA3It/gUL2EYNEZ/PcryH\r\n");
                    pem.write("fvVWsUad6yYBcf+iTYG6kgQoNsYZB7Es2RS+yk5WxF4Cgoi4F24RcN44+HYTHeSl\r\n");
                    pem.write("H2dalWVcGmaZoAw2J+vfXBJkhsgDEGo4Dgfilex7o/2zQBF7sEC546n7iAmuXeT3\r\n");
                    pem.write("veGdgkJfOpOB6zMyryBNiiiIOo8pr0d2RrzQ4aKM3rgrzUd45L1Iq9O92TNutl8R\r\n");
                    pem.write("HD0FDQbt2hIk3+IZtRYMSvIbvmUO2EzuiWHOrSMTY4iqW/kDURbgVZXTSIwBHKuv\r\n");
                    pem.write("BIOO2DqCjpo4fozFj/8UouTmuaULMCrpjDbEqROmjaWq0mZ3xEHc5BLIEF5zM5+O\r\n");
                    pem.write("FgaK+9+j1yGLK4fW2kka/qmRt3d3fv9vutoNiKmTgCiPczhSqLd5Ea/O7t+9JUR4\r\n");
                    pem.write("SRVT4DR/AuHCRBxKVZHGqEfRzXiVUlU/QJWDns6VIlWv9LfprcvRt+fcxOYjuOAs\r\n");
                    pem.write("MtR5kmPJmgb80Q+7jd5D7eXg6kQQhyXwIDsEiZ38R02F73lbXNYnBGtfCKEEt+iM\r\n");
                    pem.write("yQwWUYKoFGXW2gBQGVF+zoYbRUwDoZzScXzwMrkOk1Ef6WPjKGJbkUUzs4yUovq1\r\n");
                    pem.write("LpRp0BbV55hbsT3eLhxboXQz65cGL4dBv2DIVfmMDx2c1tblNgH877kTHjrat2q+\r\n");
                    pem.write("YnM9LFte49zbTLw0FxL0XA3BDaV5H5+DMcakkocjiaMJVqYOKfWiwdEqf+8lUIBU\r\n");
                    pem.write("K00sr9lHcbM40W4nmRIRG4Rh+9A/8RnilO3icSwSCucx4tCgTJWUGPL6G/M3o+cz\r\n");
                    pem.write("YLFMFvS89k94fOExRyqlQUJmjNjFUhMUiq/Qi456VYT3HSslJR+jESva+Ss12IEy\r\n");
                    pem.write("fChR2z9O9KJPQS/KOguFvm1rAm2A75ttQa7tuIHsp4vwXo1mBa7PIdZjFYPhSqTK\r\n");
                    pem.write("Bk2w0mzSFShlw7E9/NTvKFC8eIY3pptulJszKsxEnna+TxsOWJVtFirJlupO1bkL\r\n");
                    pem.write("mMI6JW03S7D2JOfPnmJz5j7O3LbDqjy2NMWWuhPFeQ8BoXtwTHwJfBTdD1rWI6Uf\r\n");
                    pem.write("0Oqi79QRgP/RwaWmku35T8gLWEFplbLr///PxCZ5+TSWPXNTKCCPmdPrYmbON1A1\r\n");
                    pem.write("hyXqHQKIX1+/S1h/xO4dza6snGHMmeM4nI3f44kxX4i0y3XtRJ8UH6eVYCuRHd4F\r\n");
                    pem.write("EEEZHnoxbnODTe61yx99ducZsAhzBj0M5sVr5YhAA+DQ6aDBQupxHWRi6FQ=\r\n");
                    pem.write("-----END RSA PRIVATE KEY-----");
                    try {pem.close();} catch (Exception e2) {}                
            }
            if (!((new File(serverX509CertAndKeyDir + "client.pem")).exists())) {
                    //create sample client.pem
                    pem = new PrintWriter(serverX509CertAndKeyDir + "client.pem");
                    pem.write("-----BEGIN CERTIFICATE-----\r\n");
                    pem.write("MIIFODCCAyCgAwIBAgIGASEMITfvMA0GCSqGSIb3DQEBDQUAMEExPzA9BgNVBAMM\r\n");
                    pem.write("NkZUUFNFUlZFUkNBX1dBUk5JTkdfX1BMRUFTRV9HRU5FUkFURV9ZT1VSX09XTl9f\r\n");
                    pem.write("V0FSTklORzAeFw0wOTA1MDMwMTM0MjlaFw0xNDEwMTQwMDQyNTlaMFQxUjBQBgNV\r\n");
                    pem.write("BAMMSUZUUFNFUlZFUl9DTElFTlRYNTA5Q0VSVEFOREtFWV9XQVJOSU5HX19QTEVB\r\n");
                    pem.write("U0VfR0VORVJBVEVfWU9VUl9PV05fX1dBUk5JTkcwggIiMA0GCSqGSIb3DQEBAQUA\r\n");
                    pem.write("A4ICDwAwggIKAoICAQDQ13lkkcJUzM6KSVy3w3ZlFDqedcgx4qKYOgupF+pAww1u\r\n");
                    pem.write("zaiWARzMQ72phObIXFfO7rAA+FZoeumQE88+CPIRKc8eAGxKzop7pJiAFDV0X60d\r\n");
                    pem.write("BVh49Ctj0s3y41wBtUmJTCKl4Q2HMWikRRVCBweR7UCd683cuQspYClfHSVco/Z+\r\n");
                    pem.write("3n50g8MRY3EAg1uv7qTWGupgc5fYx/1EPY2qOEcC46OVewQegNXLmk4rcqSyUTPl\r\n");
                    pem.write("r3OW9iOqMunrTtZx7qepql4rjI0+hrKZZkxEzE+OjnWWbbgx9jEoRFhkbi7MIN7A\r\n");
                    pem.write("ICUvpwt1/vWTcdqgjlpogGfc2KKZo8h91aAAV2zMMzLJmwvPuA/ATnOEX6Ku7Jxq\r\n");
                    pem.write("DxXkYxkZ9fP7Ld4NIda+k2QGrNKD71/GBzz4/DVx2hlCPuBqkfeaXe4WiFc2c4iU\r\n");
                    pem.write("3XaobldfVxCL9UNXRmzzYj6B4vCbR5QRlKcmteurzXScGhB4soTE6bercQ8g3Krk\r\n");
                    pem.write("d7IF1MXTvpIK4nF/wtcmcd5G0nTrikmYtapxmmj/u9hZUjhO1mmqOzF04flJfRH6\r\n");
                    pem.write("peY4wY0qbh5vJelCtjMutoMxjibd6WciBddmCty8jlUmzaj2xETPSqcMqgrWO016\r\n");
                    pem.write("RD5r5Ir9CAexfqPfMbTf3IyfN+O4H6ls6Zc90UttqASJsZeyZSlBH1a2PtK3TQID\r\n");
                    pem.write("AQABoyMwITAPBgNVHRMBAf8EBTADAQH/MA4GA1UdDwEB/wQEAwIB/jANBgkqhkiG\r\n");
                    pem.write("9w0BAQ0FAAOCAgEAQxSnGLWa+mlVWY2xdAdCBkVMTez+Wy/7HZ5ijTD40TB5xi06\r\n");
                    pem.write("FWSiJikSUJ7XewvBeaeK0hmJHZqSXBCsUSPj4dLQLWqwhYojUHYfyR8sqFcCrByZ\r\n");
                    pem.write("Llj5iblU+Ihpyxy3qkXvXxmkq5jQkN8ZXILUVEZ4JP2gG3vRkYuz7/KY4RGXogj4\r\n");
                    pem.write("1gphP001nSGo4E8weV+sCc79FqVoQbXufGZKM2aJWFguLO5cQWywSCFj6MZoxJOZ\r\n");
                    pem.write("u8dmJ+jcpmLgT16PvpKaYPpmKj25Y/Qu58xOWebl8lzJEs1G/gWqRqaOx8R0/Zfh\r\n");
                    pem.write("qmrBE8u8WT/JiSteMrChKUJaWpFJzJcUrxbmkYFbajBc2QB3BpgHIvNeCRQWL8Qi\r\n");
                    pem.write("uWrSc5zEK+C5wWZnewAXXb1c2xLMUNGksILBREz/42WdnF5AGbg589cpCkF+zRvV\r\n");
                    pem.write("CuFeWsFtrp8jtu3ZifdSPxIuQOXOcDf/OyCuZB1VLtG7sluuMohLa7pQoit3vL3X\r\n");
                    pem.write("YJkX63NfoPJzD8Aga2DLOxJ8zC54L1aMVQp+GyGOdImHMOeIXJ6Ka4SMCtyXDwlU\r\n");
                    pem.write("TMAlVMYcCqUdQ33QTzPkFRioQasEZJ9EP+3aLOul4CmteuPMkuNNgfQC1j89Dip4\r\n");
                    pem.write("wQw2ChzQLkpHxsjHslm6oCi/XvjhkWZExTl5zBxiSo6jh43JtxdQEuwcNNw=\r\n");
                    pem.write("-----END CERTIFICATE-----\r\n");
                    pem.write("-----BEGIN RSA PRIVATE KEY-----\r\n");
                    pem.write("Proc-Type: 4,ENCRYPTED\r\n");
                    pem.write("DEK-Info: AES-256-CFB,ce7d7b5008a266c8923ccb32d2d5f231\r\n");
                    pem.write("\r\n");
                    pem.write("EdA+7JCZKmcLN5W2R9Q6+iy4xea5n7V3iLh5hll3Mj9B+eRVzj1mPitxDDpBe6gf\r\n");
                    pem.write("LM8y3zPtn3Xl/HM31YW/JOnNPGGloUkiDb0q53fIL4b+ytqV47hr6TS5x07KT4gg\r\n");
                    pem.write("UGUD+XCigpJEKRbkWLD+IDkq57ArkNwz+SHZV/awfy7A+HL2U3goX3W2pJN8vg5S\r\n");
                    pem.write("XvU6EwoyDDMunvD3zjlSDadUWsLPfun6iWzjd4s9Z7HnPttfN9DEYBpLJaqI2IcH\r\n");
                    pem.write("hAYI1e2XeeRIJOSNnjfy5xWhjl/Zgt5U12sIKYcmsEE5cFVlmU+x0EV08/mPi8Cb\r\n");
                    pem.write("UMG7AphUy8NSfHSqJBx++ZJN5PAkmVUKJY/+zSF7SZGkXI5JJTz7tmSh5Z7PHtIo\r\n");
                    pem.write("JmJlPL2HG0gOPZD9a6ReC93ElG3MCtuzG6IyDRoZJba1yYN8XnH5T0WPxaeAmiCe\r\n");
                    pem.write("IIj/X3zgbIwJGRZAidveFtlChkpivFDOdoIsyDDPx5KX3V4kotxi5GT3A+TWVYzy\r\n");
                    pem.write("Td2Q69dEY187GRWZzwHykhGtcQVtj9ZsfXTdUsMrmjQEKk1BkR2+vAcWffT1WnZQ\r\n");
                    pem.write("DrgsVtLvxzfI46yhQ3LOm7o/XLdeFZqS/6IWRawsytVkscs+xpGSaSHOzMzhdPI3\r\n");
                    pem.write("uhBLxGPY3LEtXW1Fclv3Oq0g40oNzhhsKgqUcInCerlWulCXc3wD82HGs6avzWQx\r\n");
                    pem.write("mrWxnDNP9mvonT/ozkk8NRNO4MRHdmCac4/WMQFzJOe0wOA9MEIuNPSK3l/OFG6c\r\n");
                    pem.write("UlhprASsX9egtp2UgLjGM75KjToLuDuyGznW+fzkTYDK3iPC4xnUAGxd64SDoZvR\r\n");
                    pem.write("L2Hf7+/raq+gmea5PKsrv7S6YMKjsqYeWxpAZWaJSaJrW2mSTxv5T3bNubmMeddz\r\n");
                    pem.write("TzvLZ4jSWMowcgw/WdkAP7XXpHTvDd+6GDlogv/h7qZnJOjEn1v/0Q/DX1PJexOG\r\n");
                    pem.write("6bOJyc+5MFXtOoBe1a1mNSl5CJ7UAhSTr9qGWbaRYpZ+KRFe8+Pj6E1CmgOk/rhb\r\n");
                    pem.write("QSfebuxr2Gc3UcjaExSo/hb7b5M53Emxg/7VL8MAdfCSHZYjKdb8bztQYb7qytVH\r\n");
                    pem.write("wNVPWILzAZPuK3OuKnZhove5UnEgBj1BbFI1XZKM9M5hBrZwcgIgLy6aRLSoFGCq\r\n");
                    pem.write("bZvG73bCh/v108xS70f0UuFYxHs9Lz7AiEYvOb7uJlfdLPSK7chGc4oX8VRJQNNa\r\n");
                    pem.write("3FwpAGkix4ncyk2kyZy6N62HC/fxb+kYm9rgKcKj4bDAR+z8WNQ8JYWyf1E6K8eI\r\n");
                    pem.write("DU9HJtYQgCXCKBYW7UmYMUcUcgGe/PykzlF+eQpAzlnpOro7DeJ6EwlbDlDZAlhQ\r\n");
                    pem.write("AUEefnFuWId4BhQGdM+nUsp/YJrGEqOTANsBcIig26tWTi++sEdLF3Zp4us0sC0m\r\n");
                    pem.write("FSaB4GsfQoQ1EiAhiyfzlpxUGAecHg1tVmZFY3xj4t3g9DwA2hL+crwIVK/UplMu\r\n");
                    pem.write("aUYYclm03fzXogJ/c9jqkBtc3nspy/Tx4/ZOQvXAX1IIjayA4rcHQBtHrUpOm5b2\r\n");
                    pem.write("1CpqORD8q5l3cz0svPbA0cbvuw8fma/ofKeOJ27hpE9YUHuA60xEwLU98CnQSCe3\r\n");
                    pem.write("bLZ8timzkKDvfABbKGbnSZTHpLDVgrq3mcA8otOmhyGaDN78742I3DL8I00xHaov\r\n");
                    pem.write("dHL7+gVEvelsGmGLJfm/sDWojezmaK1bkSczl5pKXuw6m2KHbemjbF/kwgcb457O\r\n");
                    pem.write("626dDHVOnxzd8Jakq5AsSlGO2GrzfTumxNZnaBwq9JRaPGdWzhv36RBp8L/x28DW\r\n");
                    pem.write("VgFuepoWvqzUZaqKGYmtU/akm488HHniployegSOc+Yw9tGVus53WPpSlq2WWAFk\r\n");
                    pem.write("6CtfDEfctOwXpt1XW5scaxI1dNwffI3MWAa+3hJ/PIDc700WPF9CP2JLYm/gDYM8\r\n");
                    pem.write("+3FuLmPy581JmsPE/0SOVdilam464tH8tu+aqjOywKvqUUTTbi43G/1MQoM6tnSQ\r\n");
                    pem.write("AkLAFm5BJy+1l/P1F9+dzJm/Yc1EpE91AxUitD23DFkrS3c7DW+GHUymUs7ehdv3\r\n");
                    pem.write("Xn536NerwKbqHR4HwNRMKhmMZbjYTZ+QqSCJXWZMILBHKb9y4EUUZJxaYUqDap+A\r\n");
                    pem.write("jHwPSqNuclh5trJxigLmuYuddjAErTLCzRLyXI/ohvJT4U2ml2lbrfAYeziS54mS\r\n");
                    pem.write("pd85khGl36vM8Db6IwIpibdxzqD7mlD3sk4MJRBz6TNmBIrIBNNxNWdqud8+Uvkq\r\n");
                    pem.write("0Bneb/liW/HBfcUw3EdR7SOWvJEJvTtd2Ch6zM2+SGEgeQ+4pl3MiYJFQThJTOPU\r\n");
                    pem.write("qljyP46xalxSIqrPn88tcTiMY+Mv8CBX5Xwy4SWXXbjRq4UBlo/WXIKHS+Obr+1P\r\n");
                    pem.write("YuJtGqb+6gzOpGBlXLRIlIWKNVrmceW5Lne/lEiVEdeb8NcgdSnb2nQ09h3JFiY0\r\n");
                    pem.write("ySOXOFloWR3aLY32Rl2DUnE3bpU9rPS+EIQf7moTk4NDNFW+12DwfAjDAXK5xx/o\r\n");
                    pem.write("1wW2oGGrnn7KLc0NiIdptEw6KsRdzL8hrpyxnvwn2vacsfyuIHETVKjly9xRpbjk\r\n");
                    pem.write("gwEE3qluOKB83vVTVbB1DDH6FP2MBuGbyFsReS7n+U5fs2ybBp7OvphgrooZMCoZ\r\n");
                    pem.write("ru+cbkoagjgir0R6zYFVPE/2GZUxwhsFIFK0UWpuli5WSkKVkBg2+jSS905mNa8M\r\n");
                    pem.write("eSGPekrROeetzdutrg9fkHU23zOsjyERpPoCc3xZu+cRab9TgpzOb34j+QB86Wnt\r\n");
                    pem.write("dJ3h8k0i5+C+M0CqhF2oGzW3m1GM34scOBDOXGRjcawWoqr0nKalyC6koym+SICw\r\n");
                    pem.write("sryyH5AuaZKTflJ7YEdRnazdkoGVRVEvcC4leRu0u9VR9MF661L0o++pW3TwwZ7z\r\n");
                    pem.write("hFt4ZEYyPPUgj4apuj0jw8wSzeltH4LrgzKqjRGUsIw6vNNpUe5daHrSNZzsGJIM\r\n");
                    pem.write("YHBRLBUG4xrg0qaaLmO+3sMRHyrtsL2I5+BsCWxtXj5Y+SI2NZNWFjFdXgheap5p\r\n");
                    pem.write("V0DBxu3QChEZq5GMFGEl0J4wg9kpjBccoqWpsAqRvR3KLPycKpjCeau68Ufm3Rid\r\n");
                    pem.write("bLIx90hrm0N5Bev7OmctRYOKo7OjKjwIIRv1pBr/u1Is/TuZg0UlEFDojEiGPw==\r\n");
                    pem.write("-----END RSA PRIVATE KEY-----");
                    try {pem.close();} catch (Exception e2) {}
            }

            String trustedCADir = Utils.validateDirectory(TFTPServer.getAppdir() + "TrustedCAs");
            if (!(new File(trustedCADir).exists())) {(new File(trustedCADir)).mkdir();}
            if (!((new File(trustedCADir + "authority.pem")).exists())) {
                    //create sample authority.pem
                    pem = new PrintWriter(trustedCADir + "authority.pem");
                    pem.write("-----BEGIN CERTIFICATE-----\r\n");
                    pem.write("MIIFJTCCAw2gAwIBAgIGASEMHklnMA0GCSqGSIb3DQEBDQUAMEExPzA9BgNVBAMM\r\n");
                    pem.write("NkZUUFNFUlZFUkNBX1dBUk5JTkdfX1BMRUFTRV9HRU5FUkFURV9ZT1VSX09XTl9f\r\n");
                    pem.write("V0FSTklORzAeFw0wOTA1MDMwMTM0MjlaFw0xNDEwMTQwMDQyNTlaMEExPzA9BgNV\r\n");
                    pem.write("BAMMNkZUUFNFUlZFUkNBX1dBUk5JTkdfX1BMRUFTRV9HRU5FUkFURV9ZT1VSX09X\r\n");
                    pem.write("Tl9fV0FSTklORzCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAJkMk8IO\r\n");
                    pem.write("cbFauETIRPCEggCu+1/ypNUL7Y6ZreOPsjIjnD6BVO55a6/dE0Tlntwmg4Z03Eyc\r\n");
                    pem.write("jYfFrFGijp0J+UtQCsZlsnq5NAwYrAnZSfAzAuFgNqNWI3p8uui+7DtRkkNhNIZc\r\n");
                    pem.write("hEA51R/hQ6DWcj0tc7N7BAcTvsbWmbnbi+UtAr3wa9YaeOTNAEQbvPdX95nlqlGY\r\n");
                    pem.write("T6zCAHGgbxx8DM7A9yqplLS2aJuUgV/m9n18uapupER6o+ehQa/lDMTBOxGKvJxT\r\n");
                    pem.write("f9WncRvFeDpHfJfAdOV1Msz+R/A0lIKMhQhBSx5pclRBXPytRiCg3H4lPQn5Y/7J\r\n");
                    pem.write("zON8mMRL5ol+7QNwHYvqFDMeqp6/EyNtGgDzHU9X8Q3aF3tc182UE3CccC9a+Se4\r\n");
                    pem.write("/zKSBAvRCdlLvVHLC9fD7awJ9WNWTlFwyHEq3ntZKballJxj/SjbsPT0Fbj26vHY\r\n");
                    pem.write("suMRe/7n1a48nV4S+b8HGtcywpWCJjyZ4AgwcGEycyyX4eNHK8pSEr2CEin07gJL\r\n");
                    pem.write("lhOqJlMBNjODMt2eGFDZptTFCqWMHTf+c8WBfgXH8ntLvWILBAp1KVv5Hz4XDSGY\r\n");
                    pem.write("wYL3vXGw1eIGVcQKAZoxAfFOhS/BKxwE2mvz3c2Ov2548I/IVvyHNEsSipYICXSw\r\n");
                    pem.write("dCVlAFSOxQBQTathAUfE5OBdxJO5epabd3rTAgMBAAGjIzAhMA8GA1UdEwEB/wQF\r\n");
                    pem.write("MAMBAf8wDgYDVR0PAQH/BAQDAgH+MA0GCSqGSIb3DQEBDQUAA4ICAQCEC/3lvoIc\r\n");
                    pem.write("mrdnk44wo3wXbwYi8UEifftHyhO+CrkUc3X3wlpI8wnXODgNiUgBQwA0Ci3EfVHl\r\n");
                    pem.write("6WeoRNxYSjYlj7wphOJ+14nN4oOEAkudk4cpHZBshmVc2GhAOmgVgvr8lXEZzUAi\r\n");
                    pem.write("RdnuKbrBpaILR569/ou2WO3AD+UV3yIUD8Fl4SaOxS+7xiebc0StPHZ4QHlBC9oE\r\n");
                    pem.write("5F+l/JuGRWeql4UAobeCj2NqpMMtV523l1ygzNbhtkmqXMC0APkliQgMKYNRkion\r\n");
                    pem.write("AiR3COVvJeOtPXPLeh8ZeNdcwj2U6YD5QxQAjs7esH/3yzPzt4XtV/kuUP+vX8xy\r\n");
                    pem.write("iJHS/e/txkTaHtnQTIkNv1doQ8RfmbHhh+lZ9v8N47CTXVGX5IYBQY5ByIvh+zMY\r\n");
                    pem.write("O6GWXKS4dhKTkYC1Gsf6fdTHxoB0949VUgjrnqGG4uj1qRPQQydESmcHqBK1CdYe\r\n");
                    pem.write("UF0VCe1maItyuZ0TV0kg3yMk/5lOqsY+RhheFD49GoLeV5HmvrJLSEwM4jeSO7DI\r\n");
                    pem.write("JAIeZNUHGW1P4VSwUOeSYew36xAc6IIYmBu3uiV2Kymc8OCUmdbpWrx+JU/Qw8uf\r\n");
                    pem.write("eFDDI688rWCH4SO0qmTg8OY5fe+C4FCw1QtBpqzVnus7srJ1Udntu9/9/05ooRdR\r\n");
                    pem.write("UX3GOY+HlCFjKNiuM5Wn5xVLPTUSYt0N2A==\r\n");
                    pem.write("-----END CERTIFICATE-----\r\n");
//                    pem.write("-----BEGIN RSA PRIVATE KEY-----\r\n");
//                    pem.write("Proc-Type: 4,ENCRYPTED\r\n");
//                    pem.write("DEK-Info: AES-256-CFB,505d1aef2d6b063b777d98caa9cdd5df\r\n");
//                    pem.write("\r\n");
//                    pem.write("euRj/1krKSIbdOEl+uC9+6qd9lFrMbqXC6F9A70akAFQkUVfQnmhn4fziS8v5ZjI\r\n");
//                    pem.write("vFTtxlxVZuajE9p/2cnKNCnRGDcIZfdctg2ubsCFOdRfJKVcp8TelOP2bDcHZ6nw\r\n");
//                    pem.write("jL1pMVv/JtOyE4VeuJ89Z4ZivzPHwDd1Zbi7duVm5+Z9fs+EVyrf0lwomIL9Hv6T\r\n");
//                    pem.write("khfrBNu4k8XhqPJGPPAjUMdzFecY3G36sahdQwpzwTvm7bXUdosCzikRNtot+Yo6\r\n");
//                    pem.write("qsZuhRuHPkYW1IpLpPu5aOtJwmht08xvtFda5Iaff/BZNQN50AK4V15uFRWrsZBX\r\n");
//                    pem.write("crMnsvffWmeCpnHLEAKLtVfOwDCtV0lSHYe9SCJwPfli1dR5UWboCitXlENMakP5\r\n");
//                    pem.write("UPVvLc2cOQxt/u8VEaLFt5X9Zl4HDHe1CpwxKTti2P6eVE1zk8w38pZyn/rLBMmv\r\n");
//                    pem.write("Q4leDA9FKC2tGTb6MRBEGO9TGTL8ldjhv+H10IelbQuj2zlCTojgyocOVOvgF2p/\r\n");
//                    pem.write("qYAtWnN1sw5m0gPTtlWhJdXeMeGPPFnR1UvOwSHaEWxrNW0awZ21RGoJLSy4ElcH\r\n");
//                    pem.write("nNdzL/FNkNJ0YzPYiN4lRZlbhrv8abCkuPFw1sWdArgEu/CNFUJumwBfOeKgXpHS\r\n");
//                    pem.write("eP8T5agZmk1efRNQCv3tRfj25xok936En/yJS8Gx930RgCBQGvmjt8UiPqG1O6j9\r\n");
//                    pem.write("kRnWD09xkTFNrLE1NEd6NHkW2llsJjNJ5jOFkuxr2aymP03Onulw8oPMfwHOsdlV\r\n");
//                    pem.write("L/ssUGeSGWZy0kALuBRV6cTEK0TdeHxetj/ijkaTqssLeDL3/VRSyA0/aClkOJLt\r\n");
//                    pem.write("t4aBdKTV0BJ2V5/4PgcH5wGVOb/kp4s7XlrwkvbjTXp5fZzNXdhJlcRSHTSK1A3e\r\n");
//                    pem.write("8h+u+vZTKaIrO1zu67jd75KAcgquFnzh59gX3G/zkNfFZSzFgRaOHaaV984QVGaT\r\n");
//                    pem.write("nSf+2nUUZA4kqN3KltxbXAisIxJMKMSROdzqWeTXl4h7nDOILx54G4hXTs1mZfxz\r\n");
//                    pem.write("l6DNAZFWavlv+WDnn06zWsSRAEHIp7unhRpZehqWyavjpDoJ/QP0DmFze+MwfzVH\r\n");
//                    pem.write("eqTBD2Xog46cznpHv/IxCAIRiiDNtSQOvdrJXn52aIjJOobPtAlKCI5BIqyN32JC\r\n");
//                    pem.write("Htc72GVThz6sqipURVQP/QlX5S6M686UzsGDijXIVgZhePzAQ2f6SCf8ax2HWp2e\r\n");
//                    pem.write("7+YUYx5uOTXs9ARwivZ+TcOfjIZwW+priBYj6smnNVAZ3NGGS/bFx5Vw3jitgwvL\r\n");
//                    pem.write("r4Um0YfCU+InK8FyG+Nc6LqqaQ+wdM/qRV8pC/gNmPfpFP6V9hRiHMHseYKKWCvG\r\n");
//                    pem.write("XypoCLWGYssTCeABHjfplyfSnf/uEnTrsOeps6j/Ql3S5ojCVbIN6WAvDfRvwLGD\r\n");
//                    pem.write("1zHINsDhhB3Y3e0OVoCfrcd2XqCZFjfDz/Uqgg9MIgYM97yPTuCEzyCxD3NG7RKi\r\n");
//                    pem.write("tojXod9XHk99JT/lWAQUy0MjqYnIHWInFpQww9EberJ6BzROOJMO78xRANiuiJUC\r\n");
//                    pem.write("MWLKhrrwKRSr6xo+69Iur+Soii3O/g9u1jm3H4r11qxodmmjrnu7vjsj1oYrKS7+\r\n");
//                    pem.write("rsS1hKzujzKFYa3CzLHzVspGjfFfwX007qL2qxBYqrWYRcNBVO9v9k4wMCBAq3Uy\r\n");
//                    pem.write("bce6jIRSGEkFxjZ3XCmUDkNnie5nxIs3GBke2nQl5e4iMTtrPZzGmev5n6UNS46N\r\n");
//                    pem.write("GdPWYymh7vl3rSr63x5TSthmzWDMbk2xRsgHhMtquXFQqMdByfY78qnWebdNgEkO\r\n");
//                    pem.write("XA4iGnZLplVTUBb5TgqbyeZbfmTjrt3e9JHkR24xiy/EP4kEJ6NAcHrr5aNvHyBX\r\n");
//                    pem.write("bTKvH5MvB7fYORyLRGCvOIxuwiXyJG4nuY0E2fSNwcqez/04ng1phVzHPPafx1pH\r\n");
//                    pem.write("QrsIoLGMu0qsmRASn7VYR0IvTiY+TZHXbBWoliVkgziYofa+IyVaITsQw/9uz6mz\r\n");
//                    pem.write("J5Fiep0pnpVZDgWIa637GUa7KcotAqIFTS82H4JBAbc9lWGIzBP4bIqy2HTLSJcz\r\n");
//                    pem.write("67f7sPcPfy0+F+kv1fdIIsb20g9II65K36L9D2kL3ocXi+VMOvZRMePyNohs8htS\r\n");
//                    pem.write("1xTpgl0w/oRpLbdPCuexvpdX45IHXWzn6EVvpZ/ipiGYkrbiUIsjOGq64XA4Tk2/\r\n");
//                    pem.write("vpOY8tkuuW9Kw5hRUljmSbJS/gQS5XHOJyqsFUCT27XVeQOSLqohvGEC1fzd/5Vj\r\n");
//                    pem.write("7oh90bWVfgbS0J4uq2YeRczPVyPPNZ6pLtCq+cwog/LIv5p7/Df3bFDO4prbuio4\r\n");
//                    pem.write("37aY1Wfo35qBUPxKXA9lmkC4t49kDnauW5IUbkTfQL6010+cEcog8IyEvOSYIxUi\r\n");
//                    pem.write("uEGpERBYVnDV/xKpw8m3HgAbR2bNOuFQHkwfwtBvmHvQqQwoG6McMpaQJIsW9Po+\r\n");
//                    pem.write("3AEueAMq3qobRlu/6rBtqHVu4mv3ot42Q+HTNuMnzy+uQOHHxaIAHBds9uo0GaEA\r\n");
//                    pem.write("ml0OlIwAML8Nypk7vcoyTD/dICoc7+aqz71kWggAODyvUc04gO2fDdpDDZYmbzTU\r\n");
//                    pem.write("5CIiP7kjfDPF43cSu0Sk8/6Xzqmb2yncAk5dWcZA5dF5qaMColZwk0Rib72soDLi\r\n");
//                    pem.write("1v5VQTADgIiLAG4mgqhy3SuLvWZsqiJBGQHcY98qsfnGnpl1gUA9JAfTcqyABoAw\r\n");
//                    pem.write("vZHa/yJZK3RFZOfJFd3AMJHOtOkpLAfpfF1PLL5K3Pl7eLYG0HN3l8fhYVUhj1qF\r\n");
//                    pem.write("TazckpOU6KbzNmKi6H21JYtCHJI53Ho/HPOgGP8qXTfbmYkb4nkomOGLjhPMgDHx\r\n");
//                    pem.write("6jxo+An7USqB/b2nW49PBBTearpYBTaLZ1rd3FMuJ/h2Ub2KdSclC3Kfl2jRqxz6\r\n");
//                    pem.write("3TnYlAvsYJjmwb4ZdWEX+ebQ/rXuQP85MarUqPXXhhbxq7h1YH5RugIIUlN70fyO\r\n");
//                    pem.write("uCjxIyir8piF3IKZ4uH9jeQ8adtNVXpRuAfg1sSvFWl5HBe7FrR53Uk2i7RYyh1/\r\n");
//                    pem.write("/P4bQvkiSt6Rud2gv038I2FMYIj1vtKmSvKdQMBAmPFWXb2jbYnXL2APsOP9/iBo\r\n");
//                    pem.write("z8YTg/whiHrXF0hEOqoZy/JQPYGOAGt2wu+fnuKDMQ1VheNc0GmMS/11ng==\r\n");
//                    pem.write("-----END RSA PRIVATE KEY-----");
                    try {pem.close();} catch (Exception e2) {}
            }
        } catch (Exception e) {} finally {try {pem.close();} catch (Exception e2) {}}
    }

    /**
     * The method that starts the TFTPServer. It reads its settings from
     * {SYSTEMUSERDIR}/.tcpconns/ftpserver/ftpserver.conf; if this file is not present it
     * will create a template
     * @param args
     */
    public static void main(String[] args) {
        TFTPServer myTFTPServer = new TFTPServer();
        try {       
            boolean startServer = true;

            File configurationFile = new File(TFTPServer.getConfigurationFileName());
            if (configurationFile.exists() == false) {
                startServer = false;
                writeSampleCerts();
                createEmptySettingsFile();
            }
            if ((configurationFile.exists()) && (startServer)) {

                myTFTPServer.loadSettings();

                System.out.println("Starting server...");

                myTFTPServer.activate("0.0.0.0", myTFTPServer.data().getServerPort());
                int t = System.in.read();
            }          
        } catch (Exception e) {
            System.out.println("ERROR " + e);
        } finally {
            try {myTFTPServer.stopLogging();} catch (Exception e2) {}
            try {myTFTPServer.deactivateNow();} catch (Exception e2) {}
        }
    }
}
