/* ============================================================================
 * Dialogs.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2008-2009
 * by J.H.H.C. van der Meijs (info@jhhcvandermeijs.nl)
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or 
 *      its contributors may be used to endorse or promote products derived from 
 *      this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS 
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 * 
 * THE FOLLOWING CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON 
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES 
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE 
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE 
 * BELOW VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE 
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON 
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 * 
 * ============================================================================
 * 
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 * 
 * ============================================================================
 */
package nl.jhhcvandermeijs.ftp.ftpserver;

import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JTextField;

/**
 * Helper class that takes a JTextField as an argument. If the JTextField 
 * contains a valid FileName, the respective file will be opened as selected
 * in the Dialogs dialog. When a file is selected it is stored into the JTextField
 * container. 
 */
public class Dialogs {
    static void openDialog(JTextField aTextField) {
        Dialogs.Dialog(aTextField, false, JFileChooser.OPEN_DIALOG, null);
    }

    static void openDialog(JTextField aTextField, boolean directoriesOnly) {
        Dialogs.Dialog(aTextField, directoriesOnly, JFileChooser.OPEN_DIALOG, null);
    }

    static void openDialog(JTextField aTextField, String title) {
        Dialogs.Dialog(aTextField, false, JFileChooser.OPEN_DIALOG, title);
    }

    static void openDialog(JTextField aTextField, boolean directoriesOnly, String title) {
        Dialogs.Dialog(aTextField, directoriesOnly, JFileChooser.OPEN_DIALOG, title);
    }

    /**
     * Opens a dialog that enables to browse for a file.
     * 
     * @param aTextField if the aTextField parameter contains a valid filename
     * it will be opened by default in the Dialogs dialog; when a file is selected
     * the selected file will be stored in the aTextField container
     */
    static void Dialog(JTextField aTextField, boolean directoriesOnly, int dialogType, String title) {
        JFileChooser aJFileChooser = new JFileChooser();

        aJFileChooser.setDialogType(dialogType);
        if (title != null) {
            aJFileChooser.setDialogTitle(title);
        }

        if (directoriesOnly) {
            aJFileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        }

        File aDir = null;
        try {
            aDir = new File(aTextField.getText());
        } catch (Exception e) {}
        if (aDir != null) {            
            aJFileChooser.setCurrentDirectory(aDir);
        }
        int returnValue;        
        returnValue = aJFileChooser.showOpenDialog(null);
        if (returnValue == JFileChooser.APPROVE_OPTION) {
            try {
                aTextField.setText(aJFileChooser.getSelectedFile().getCanonicalPath());
            } catch (Exception e) {}
        }                    
    }
}
