/* ============================================================================
 * MessageDigestDialog.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.keys;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.MessageDigest;
import javax.swing.JFrame;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;

public class MessageDigestDialog extends javax.swing.JDialog {
    public static final long serialVersionUID = 5L;

    /** Creates new form MessageDigestDialog */
    public MessageDigestDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }

    public static void showDialog(java.awt.Frame parent) {
        MessageDigestDialog dialog = new MessageDigestDialog(parent, true);
        dialog.setLocationRelativeTo(parent);
        dialog.setVisible(true);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        lblAction = new javax.swing.JLabel();
        comboAction = new javax.swing.JComboBox();
        lblFilenameOrText = new javax.swing.JLabel();
        edtFileNameOrText = new javax.swing.JTextField();
        btnChooseFileName = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        edtMessageDigests = new javax.swing.JTextArea();
        btnCalculateMessageDigest = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Calculate Message Digest (Hash)");

        lblAction.setText("Action");

        comboAction.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "file", "text" }));

        lblFilenameOrText.setText("Filename or text");

        btnChooseFileName.setText("Choose file");
        btnChooseFileName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnChooseFileNameActionPerformed(evt);
            }
        });

        edtMessageDigests.setColumns(18);
        edtMessageDigests.setRows(5);
        jScrollPane1.setViewportView(edtMessageDigests);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 179, Short.MAX_VALUE)
        );

        btnCalculateMessageDigest.setText("Calculate");
        btnCalculateMessageDigest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCalculateMessageDigestActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblAction)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, lblFilenameOrText))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(edtFileNameOrText, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 260, Short.MAX_VALUE)
                    .add(comboAction, 0, 260, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(btnChooseFileName, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnCalculateMessageDigest, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 172, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblAction)
                    .add(comboAction, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lblFilenameOrText)
                    .add(edtFileNameOrText, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnCalculateMessageDigest)
                    .add(btnChooseFileName))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnChooseFileNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnChooseFileNameActionPerformed
        String fileName = KeyStoreHelper.openDialog(((JFrame)(this.getParent())), null);
        if (fileName != null) {
            this.edtFileNameOrText.setText(fileName);
        }
    }//GEN-LAST:event_btnChooseFileNameActionPerformed

    private void btnCalculateMessageDigestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCalculateMessageDigestActionPerformed
        KeyStoreHelper.assertProviders();
        this.edtMessageDigests.setText("");
        if (this.comboAction.getSelectedIndex() == 0) {
            //file
            InputStream inputStream = null;
            try {
                try {
                    inputStream = new FileInputStream(this.edtFileNameOrText.getText());
                    MessageDigest[] digests = new MessageDigest[15];
                    try {digests[ 0] = MessageDigest.getInstance("SHA-512", "BC");} catch (Exception e) {}
                    try {digests[ 1] = MessageDigest.getInstance("SHA-256", "BC");} catch (Exception e) {}
                    try {digests[ 2] = MessageDigest.getInstance("SHA1");} catch (Exception e) {}
                    try {digests[ 3] = MessageDigest.getInstance("MD5");} catch (Exception e) {}
                    try {digests[ 4] = MessageDigest.getInstance("Whirlpool", "BC");} catch (Exception e) {}
                    try {digests[ 5] = MessageDigest.getInstance("GOST3411", "BC");} catch (Exception e) {}
                    try {digests[ 6] = MessageDigest.getInstance("RipeMD128", "BC");} catch (Exception e) {}
                    try {digests[ 7] = MessageDigest.getInstance("RipeMD160", "BC");} catch (Exception e) {}
                    try {digests[ 8] = MessageDigest.getInstance("RipeMD256", "BC");} catch (Exception e) {}
                    try {digests[ 9] = MessageDigest.getInstance("RipeMD320", "BC");} catch (Exception e) {}
                    try {digests[10] = MessageDigest.getInstance("SHA-224", "BC");} catch (Exception e) {}
                    try {digests[11] = MessageDigest.getInstance("SHA-384", "BC");} catch (Exception e) {}
                    try {digests[12] = MessageDigest.getInstance("Tiger", "BC");} catch (Exception e) {}
                    try {digests[13] = MessageDigest.getInstance("MD4", "BC");} catch (Exception e) {}
                    try {digests[14] = MessageDigest.getInstance("MD2", "BC");} catch (Exception e) {}
                    
                    byte[] buffer = new byte[4096];
                    int bytesRead = inputStream.read(buffer);
                    while (bytesRead > 0) {
                        int i = 1-1; int n = digests.length;
                        while (i <= (n-1)) {
                            digests[i].update(buffer, 0, bytesRead);                            
                            i = i + 1;
                        }
                        bytesRead = inputStream.read(buffer);
                    }

                    this.edtMessageDigests.setText("Message digests for \"" + this.edtFileNameOrText.getText() + "\" \n\r");

                    byte[] digest = null;
                    int i = 1-1; int n = digests.length;
                    while (i <= (n-1)) {
                        digest = digests[i].digest();
                        this.edtMessageDigests.append("\n\r" + digests[i].getAlgorithm() + ":  " + Utils.toHex(digest, true));
                        i = i + 1;
                    }

                } finally {
                    try {inputStream.close();} catch (Exception e2) {}
                }
            } catch (Exception e) {System.out.println(e);}

        } else if (this.comboAction.getSelectedIndex() == 1) {
            //text
            DataOutputStream dataOutputStream = null;
            InputStream inputStream = null;
            try {
                try {
                    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                    dataOutputStream = new DataOutputStream(byteArrayOutputStream);
                    dataOutputStream.writeUTF(this.edtFileNameOrText.getText());
                    dataOutputStream.flush();
                    byteArrayOutputStream.flush();
 
                    inputStream = new ByteArrayInputStream((byteArrayOutputStream).toByteArray());

                    MessageDigest[] digests = new MessageDigest[15];
                    try {digests[ 0] = MessageDigest.getInstance("SHA-512", "BC");} catch (Exception e) {}
                    try {digests[ 1] = MessageDigest.getInstance("SHA-256", "BC");} catch (Exception e) {}
                    try {digests[ 2] = MessageDigest.getInstance("SHA1");} catch (Exception e) {}
                    try {digests[ 3] = MessageDigest.getInstance("MD5");} catch (Exception e) {}
                    try {digests[ 4] = MessageDigest.getInstance("Whirlpool", "BC");} catch (Exception e) {}
                    try {digests[ 5] = MessageDigest.getInstance("GOST3411", "BC");} catch (Exception e) {}
                    try {digests[ 6] = MessageDigest.getInstance("RipeMD128", "BC");} catch (Exception e) {}
                    try {digests[ 7] = MessageDigest.getInstance("RipeMD160", "BC");} catch (Exception e) {}
                    try {digests[ 8] = MessageDigest.getInstance("RipeMD256", "BC");} catch (Exception e) {}
                    try {digests[ 9] = MessageDigest.getInstance("RipeMD320", "BC");} catch (Exception e) {}
                    try {digests[10] = MessageDigest.getInstance("SHA-224", "BC");} catch (Exception e) {}
                    try {digests[11] = MessageDigest.getInstance("SHA-384", "BC");} catch (Exception e) {}
                    try {digests[12] = MessageDigest.getInstance("Tiger", "BC");} catch (Exception e) {}
                    try {digests[13] = MessageDigest.getInstance("MD4", "BC");} catch (Exception e) {}
                    try {digests[14] = MessageDigest.getInstance("MD2", "BC");} catch (Exception e) {}

                    byte[] buffer = new byte[4096];
                    int bytesRead = inputStream.read(buffer);
                    while (bytesRead > 0) {
                        int i = 1-1; int n = digests.length;
                        while (i <= (n-1)) {
                            digests[i].update(buffer, 0, bytesRead);                            
                            i = i + 1;
                        }
                        bytesRead = inputStream.read(buffer);
                    }

                    this.edtMessageDigests.setText("Message digests for \"" + this.edtFileNameOrText.getText() + "\" \n\r");

                    byte[] digest = null;
                    int i = 1-1; int n = digests.length;
                    while (i <= (n-1)) {
                        digest = digests[i].digest();
                        this.edtMessageDigests.append("\n\r" + digests[i].getAlgorithm() + ":  " + Utils.toHex(digest, true));

                        i = i + 1;
                    }

                } finally {
                    try {inputStream.close();} catch (Exception e2) {}
                    try {dataOutputStream.close();} catch (Exception e2) {}
                }
            } catch (Exception e) {System.out.println(e);}
        }
        this.edtMessageDigests.selectAll();
    }//GEN-LAST:event_btnCalculateMessageDigestActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCalculateMessageDigest;
    private javax.swing.JButton btnChooseFileName;
    private javax.swing.JComboBox comboAction;
    private javax.swing.JTextField edtFileNameOrText;
    private javax.swing.JTextArea edtMessageDigests;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lblAction;
    private javax.swing.JLabel lblFilenameOrText;
    // End of variables declaration//GEN-END:variables

}
