/* ============================================================================
 * X509PropertiesDialog.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.keys;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import java.security.spec.RSAPublicKeySpec;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper.DNProcessor;
import nl.jhhcvandermeijs.tcpconns.pgp.PGPHelper;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;

/**
 *
 * @author username
 */
public class X509PropertiesDialog extends javax.swing.JDialog {
    public static final long serialVersionUID = 4L;

    X509Certificate[] fChain = null;
    PrivateKey fPrivateKey = null;

    public static void showDialog(java.awt.Frame parent, X509Certificate x509) {
        X509PropertiesDialog.showDialog(parent, x509, null);
    }

    public static void showDialog(java.awt.Frame parent, X509Certificate[] chain) {
        X509PropertiesDialog.showDialog(parent, chain, null);
    }        
    
    public static void showDialog(java.awt.Frame parent, X509Certificate x509, PrivateKey privateKey) {
        X509Certificate[] chain = new X509Certificate[1];
        chain[0] = x509;
        X509PropertiesDialog.showDialog(parent, chain, privateKey);
    }

    public static void showDialog(java.awt.Frame parent, X509Certificate[] chain, PrivateKey privateKey) {        
        X509PropertiesDialog x509PropertiesDialog = new X509PropertiesDialog(parent, true);
        x509PropertiesDialog.setLocationRelativeTo(parent);
        x509PropertiesDialog.fChain = chain;
        x509PropertiesDialog.fPrivateKey = privateKey;
        x509PropertiesDialog.updateX509PropertiesTree(chain, privateKey, x509PropertiesDialog.x509PropertiesTree);
        x509PropertiesDialog.setVisible(true);
    }

    public void updateX509PropertiesTree(X509Certificate[] chain, PrivateKey privateKey, JTree jTree) {

        DefaultMutableTreeNode root = new DefaultMutableTreeNode(); root.setUserObject("v" + chain[0].getVersion() + " X.509 " + chain[0].getPublicKey().getAlgorithm() + " (Serial no.: " + Utils.toHex(chain[0].getSerialNumber().toByteArray(), true) + ")");

        if (chain[0].getSigAlgName().equals("1.2.840.113549.1.1.10")) {
            DefaultMutableTreeNode signatureType = new DefaultMutableTreeNode(); signatureType.setUserObject("Signature Algorithm:  RSASSA_PSS");
                root.add(signatureType);
        } else {
            DefaultMutableTreeNode signatureType = new DefaultMutableTreeNode(); signatureType.setUserObject("Signature Algorithm:  " + chain[0].getSigAlgName());
                root.add(signatureType);
        }

        DefaultMutableTreeNode keySize = new DefaultMutableTreeNode();
            if (chain[0].getPublicKey().getAlgorithm().toUpperCase().contains("RSA")) {
                try {
                    KeyFactory keyFactory = KeyFactory.getInstance("RSA");
                    RSAPublicKeySpec rsaPublicKeySpec = ((RSAPublicKeySpec)(keyFactory.getKeySpec(chain[0].getPublicKey(), RSAPublicKeySpec.class)));
                    BigInteger bigInteger = rsaPublicKeySpec.getModulus();
                    keySize.setUserObject("Keysize (in bits): " + bigInteger.toString(2).length());
                } catch (Exception e) {
                    keySize.setUserObject("Keysize: unknown keysize");
                }
            }
            root.add(keySize);

        if (privateKey != null) {
            DefaultMutableTreeNode privkey = new DefaultMutableTreeNode(); privkey.setUserObject("Private Key:  present");
            root.add(privkey);
        } else {
            DefaultMutableTreeNode privkey = new DefaultMutableTreeNode(); privkey.setUserObject("Private Key:  n/a");
            root.add(privkey);
        }

        if (KeyStoreHelper.isSelfSigned(chain[0])) {
            DefaultMutableTreeNode selfsigned = new DefaultMutableTreeNode(); selfsigned.setUserObject("Self-signed:  true");
            root.add(selfsigned);
        } else {
            DefaultMutableTreeNode selfsigned = new DefaultMutableTreeNode(); selfsigned.setUserObject("Self-signed:  false");
            root.add(selfsigned);
        }

        int pathDepth = chain[0].getBasicConstraints();
        if (pathDepth == -1) {
            DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  false");
            root.add(isCA);
        } else {
            if (pathDepth == Integer.MAX_VALUE) {
                DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  true (path depth: unlimited)");
                root.add(isCA);
            } else {
                DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  true (path depth: " + pathDepth + ")");
                root.add(isCA);
            }
        }

        DNProcessor dn = new DNProcessor(chain[0]);        
        this.setTitle("Properties of \"" + dn.getCommonName() + "\"");
        DefaultMutableTreeNode subject = new DefaultMutableTreeNode(); subject.setUserObject("Issued To");
            DefaultMutableTreeNode subjectCN = new DefaultMutableTreeNode(); subjectCN.setUserObject("Common Name:  " + dn.getCommonName()); subject.add(subjectCN);
            DefaultMutableTreeNode subjectOU = new DefaultMutableTreeNode(); subjectOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); subject.add(subjectOU);
            DefaultMutableTreeNode subjectO  = new DefaultMutableTreeNode(); subjectO.setUserObject("Organization:  " + dn.getOrganization()); subject.add(subjectO);
            DefaultMutableTreeNode subjectL  = new DefaultMutableTreeNode(); subjectL.setUserObject("Locality:  " + dn.getLocality()); subject.add(subjectL);
            DefaultMutableTreeNode subjectST = new DefaultMutableTreeNode(); subjectST.setUserObject("State or Province:  " + dn.getStateOrProvince()); subject.add(subjectST);
            DefaultMutableTreeNode subjectC  = new DefaultMutableTreeNode(); subjectC.setUserObject("Country:  " + dn.getCountry()); subject.add(subjectC);
            DefaultMutableTreeNode subjectUID = new DefaultMutableTreeNode(); subjectUID.setUserObject("Unique ID: " + dn.getUID()); subject.add(subjectUID);
            DefaultMutableTreeNode subjectEmail = new DefaultMutableTreeNode(); subjectEmail.setUserObject("Email: " + dn.getEmailAddress()); subject.add(subjectEmail);
            DefaultMutableTreeNode subjectWebsite = new DefaultMutableTreeNode(); subjectWebsite.setUserObject("Website: " + dn.getURI()); subject.add(subjectWebsite);
            DefaultMutableTreeNode subjectIP = new DefaultMutableTreeNode(); subjectIP.setUserObject("IPAddress: " + dn.getIPAddress()); subject.add(subjectIP);
            root.add(subject);

        dn = new DNProcessor(chain[0].getIssuerDN().getName());
        DefaultMutableTreeNode issuer = new DefaultMutableTreeNode(); issuer.setUserObject("Issued By");
            DefaultMutableTreeNode issuerCN = new DefaultMutableTreeNode(); issuerCN.setUserObject("Common Name:  " + dn.getCommonName()); issuer.add(issuerCN);
            DefaultMutableTreeNode issuerOU = new DefaultMutableTreeNode(); issuerOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); issuer.add(issuerOU);
            DefaultMutableTreeNode issuerO  = new DefaultMutableTreeNode(); issuerO.setUserObject("Organization:  " + dn.getOrganization()); issuer.add(issuerO);
            DefaultMutableTreeNode issuerL  = new DefaultMutableTreeNode(); issuerL.setUserObject("Locality:  " + dn.getLocality()); issuer.add(issuerL);
            DefaultMutableTreeNode issuerST = new DefaultMutableTreeNode(); issuerST.setUserObject("State or Province:  " + dn.getStateOrProvince()); issuer.add(issuerST);
            DefaultMutableTreeNode issuerC  = new DefaultMutableTreeNode(); issuerC.setUserObject("Country:  " + dn.getCountry()); issuer.add(issuerC);
            DefaultMutableTreeNode issuerUID = new DefaultMutableTreeNode(); issuerUID.setUserObject("Unique ID: " + dn.getUID()); issuer.add(issuerUID);
            root.add(issuer);

        if (chain.length > 1) {
            int i = 1-1+1; int n = chain.length-1;
            while (i <= (n-1)) {
                if (KeyStoreHelper.isSelfSigned(chain[i]) == false) {
                    dn = new DNProcessor(chain[i].getIssuerDN().getName());
                    issuer = new DefaultMutableTreeNode(); issuer.setUserObject("Issued By");
                        issuerCN = new DefaultMutableTreeNode(); issuerCN.setUserObject("Common Name:  " + dn.getCommonName()); issuer.add(issuerCN);
                        issuerOU = new DefaultMutableTreeNode(); issuerOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); issuer.add(issuerOU);
                        issuerO  = new DefaultMutableTreeNode(); issuerO.setUserObject("Organization:  " + dn.getOrganization()); issuer.add(issuerO);
                        issuerL  = new DefaultMutableTreeNode(); issuerL.setUserObject("Locality:  " + dn.getLocality()); issuer.add(issuerL);
                        issuerST = new DefaultMutableTreeNode(); issuerST.setUserObject("State or Province:  " + dn.getStateOrProvince()); issuer.add(issuerST);
                        issuerC  = new DefaultMutableTreeNode(); issuerC.setUserObject("Country:  " + dn.getCountry()); issuer.add(issuerC);
                        issuerUID = new DefaultMutableTreeNode(); issuerUID.setUserObject("Unique ID: " + dn.getUID()); issuer.add(issuerUID);
                        root.add(issuer);
                }
                i = i + 1;
            }
        }

        DefaultMutableTreeNode validityPeriod = new DefaultMutableTreeNode(); validityPeriod.setUserObject("Validity Period");
            DefaultMutableTreeNode notBefore = new DefaultMutableTreeNode(); notBefore.setUserObject("Not Before:  " + chain[0].getNotBefore()); validityPeriod.add(notBefore);
            DefaultMutableTreeNode notAfter = new DefaultMutableTreeNode(); notAfter.setUserObject("Not After:  " + chain[0].getNotAfter()); validityPeriod.add(notAfter);
            DefaultMutableTreeNode currentlyValid = new DefaultMutableTreeNode();
            try {
                chain[0].checkValidity();
                currentlyValid.setUserObject("Currently *within* validity period".toUpperCase());
            } catch (CertificateExpiredException e) {
                currentlyValid.setUserObject("N.B.(!): Certificate *is no longer valid*".toUpperCase());
                this.setTitle("N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" HAS EXPIRED!");
                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" has EXPIRED!", "Certificate Validity Warning", JOptionPane.WARNING_MESSAGE);
            } catch (CertificateNotYetValidException e2) {
                currentlyValid.setUserObject("N.B.(!): Certificate *not yet valid*".toUpperCase());
                this.setTitle("N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" IS NOT(!) YET VALID");
                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" is NOT(!) YET VALID", "Certificate Validity Warning", JOptionPane.WARNING_MESSAGE);
            }                                
            validityPeriod.add(currentlyValid);
            root.add(validityPeriod);

        DefaultMutableTreeNode pubkey = new DefaultMutableTreeNode();
        pubkey.setUserObject("Public key:  " + chain[0].getPublicKey().toString());
        root.add(pubkey);

        DefaultMutableTreeNode signature = new DefaultMutableTreeNode();
        signature.setUserObject("Signature: " + Utils.toHex(chain[0].getSignature(), true));
        root.add(signature);

        try {
            boolean[] keyUsage = chain[0].getKeyUsage();
            if (keyUsage != null) {
                DefaultMutableTreeNode keyUsageNode = new DefaultMutableTreeNode();
                //keyUsageNode.setUserObject("Key Usage");
                keyUsageNode.setUserObject("Key Usage (" + Utils.toHex(chain[0].getExtensionValue("2.5.29.15"), true) + ")");
                if (keyUsage[0] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("digitalSignature: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("digitalSignature: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[1] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("nonRepudiation: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("nonRepudiation: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[2] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyEncipherment: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyEncipherment: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[3] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("dataEncipherment: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("dataEncipherment: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[4] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyAgreement: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyAgreement: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[5] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyCertSign: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyCertSign: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[6] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("cRLSign: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("cRLSign: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[7] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("encipherOnly: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("encipherOnly: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[8] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("decipherOnly: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("decipherOnly: NO");
                    keyUsageNode.add(ku);
                }
                root.add(keyUsageNode);
            }
        } catch (Exception e) {}

        DefaultMutableTreeNode fingerprints = new DefaultMutableTreeNode(); fingerprints.setUserObject("Fingerprints");
            KeyStoreHelper.assertProviders();
            try {
                byte[] encodedCert = chain[0].getEncoded();
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-512", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-512:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-256", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-256:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA1");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA1:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD5");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD5:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("Whirlpool", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("Whirlpool:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("GOST3411", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("GOST3411:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD128", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD128:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD160", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD160:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD256", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD256:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD320", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD320:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-224", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-224:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-384", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-384:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("Tiger", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("Tiger:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD4", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD4:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD2", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD2:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
            root.add(fingerprints);
            } catch (Exception e2) {}

        try {
        DefaultMutableTreeNode pgpData = new DefaultMutableTreeNode(); pgpData.setUserObject("PGP");
        DefaultMutableTreeNode pgpKeyID = new DefaultMutableTreeNode(); pgpKeyID.setUserObject("PGP KeyID:  " + PGPHelper.getPGPKeyID(chain[0])); pgpData.add(pgpKeyID);
        DefaultMutableTreeNode pgpFingerprint = new DefaultMutableTreeNode(); pgpFingerprint.setUserObject("PGP Fingerprint:  " + PGPHelper.getPGPFingerprintAsHexEncodedString(chain[0], true)); pgpData.add(pgpFingerprint);
            root.add(pgpData);
        } catch (Exception e) {}

        DefaultTreeModel defaultTreeModel = new DefaultTreeModel(root);
        jTree.setModel(defaultTreeModel);

        int i = 1-1; int n = root.getChildCount();
        while (i <= (n-1)) {
            jTree.expandPath(new TreePath(defaultTreeModel.getPathToRoot(root.getChildAt(i))));
            int j = 1-1; int m = root.getChildAt(i).getChildCount();
            while (j <= (m-1)) {
                jTree.expandPath(new TreePath(defaultTreeModel.getPathToRoot(root.getChildAt(i).getChildAt(j))));
                j = j + 1;
            }
            i = i + 1;
        }
    }

    /** Creates new form X509PropertiesDialog */
    public X509PropertiesDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnOK = new javax.swing.JButton();
        btnSaveAs = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        x509PropertiesTree = new javax.swing.JTree();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("X509 Certificate Properties");

        btnOK.setText("  OK  ");
        btnOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOKActionPerformed(evt);
            }
        });

        btnSaveAs.setText("Save As...");
        btnSaveAs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSaveAsActionPerformed(evt);
            }
        });

        jScrollPane1.setViewportView(x509PropertiesTree);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 379, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(btnSaveAs)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnOK)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 388, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnOK)
                    .add(btnSaveAs))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOKActionPerformed
        this.dispose();
    }//GEN-LAST:event_btnOKActionPerformed

    private void btnSaveAsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveAsActionPerformed
        boolean exportPrivateKey = false;
        if (this.fPrivateKey != null) {
            int choice = JOptionPane.showConfirmDialog(((JFrame)(this.getParent())), "Do you wish to include the private key as well?", "Export Private Key??", JOptionPane.YES_NO_OPTION);
            if (choice == JOptionPane.YES_OPTION) {
                exportPrivateKey = true;
            }
        }

        KeyStore keyStore = KeyStoreHelper.newEmptyKeyStore();
        SecureRandom secureRandom = new SecureRandom();
        byte[] buffer = new byte[64];
        secureRandom.nextBytes(buffer);
        String string = new String(buffer);
        char[] keyStorePassphrase = string.toCharArray();

        if (exportPrivateKey == true) {
            try {
                keyStore.setKeyEntry(KeyStoreHelper.getAlias(this.fChain[0]), this.fPrivateKey, keyStorePassphrase, this.fChain);
            } catch (Exception e) {}
            String[] keyStoreTypes = new String[7];
            keyStoreTypes[0] = "PKCS#12 keystore new type (.pfx/.p12)";
            keyStoreTypes[6] = "PKCS#12 keystore old type (.pfx/.p12)";
            keyStoreTypes[1] = "Java Key Store (.jks/.ts)";
            keyStoreTypes[2] = "BouncyCastle.org Key Store (.bks/.bts)";
            keyStoreTypes[3] = "UBER (3DES/TwoFish) formatted keystore (.uber)";
            keyStoreTypes[4] = "JCEKS formatted keystore (.jceks)";
            keyStoreTypes[5] = "OpenSSL PEM formatted (.pem)";

            String[] pemEncryptionTypes = new String[37];
            pemEncryptionTypes[ 0] = "AES-128-CBC";
            pemEncryptionTypes[ 1] = "AES-128-CFB";
            pemEncryptionTypes[ 2] = "AES-128-ECB";
            pemEncryptionTypes[ 3] = "AES-128-OFB";
            pemEncryptionTypes[ 4] = "AES-192-CBC";
            pemEncryptionTypes[ 5] = "AES-192-CFB";
            pemEncryptionTypes[ 6] = "AES-192-ECB";
            pemEncryptionTypes[ 7] = "AES-192-OFB";
            pemEncryptionTypes[ 8] = "AES-256-CBC";
            pemEncryptionTypes[ 9] = "AES-256-CFB";
            pemEncryptionTypes[10] = "AES-256-ECB";
            pemEncryptionTypes[11] = "AES-256-OFB";
            pemEncryptionTypes[12] = "BF-CBC";
            pemEncryptionTypes[13] = "BF-CFB";
            pemEncryptionTypes[14] = "BF-ECB";
            pemEncryptionTypes[15] = "BF-OFB";
            pemEncryptionTypes[16] = "DES-CBC";
            pemEncryptionTypes[17] = "DES-CFB";
            pemEncryptionTypes[18] = "DES-ECB";
            pemEncryptionTypes[19] = "DES-OFB";
            pemEncryptionTypes[20] = "DES-EDE";
            pemEncryptionTypes[21] = "DES-EDE-CBC";
            pemEncryptionTypes[22] = "DES-EDE-CFB";
            pemEncryptionTypes[23] = "DES-EDE-ECB";
            pemEncryptionTypes[24] = "DES-EDE-OFB";
            pemEncryptionTypes[25] = "DES-EDE3";
            pemEncryptionTypes[26] = "DES-EDE3-CBC";
            pemEncryptionTypes[27] = "DES-EDE3-CFB";
            pemEncryptionTypes[28] = "DES-EDE3-ECB";
            pemEncryptionTypes[29] = "DES-EDE3-OFB";
            pemEncryptionTypes[30] = "RC2-CBC";
            pemEncryptionTypes[31] = "RC2-CFB";
            pemEncryptionTypes[32] = "RC2-ECB";
            pemEncryptionTypes[33] = "RC2-OFB";
            pemEncryptionTypes[34] = "RC2-40-CBC";
            pemEncryptionTypes[35] = "RC2-64-CBC";
            pemEncryptionTypes[36] = "NO_ENCRYPTION";

            String keyStoreType = ((String)(JOptionPane.showInputDialog(((JFrame)(this.getParent())), "Please select the desired keystore type", "Select keystore format", JOptionPane.QUESTION_MESSAGE, null, keyStoreTypes, keyStoreTypes[5])));
            if (keyStoreType != null) {
                String pemEncryptionType = null;
                char[] passphrase = null;
                if (keyStoreType.equals(keyStoreTypes[5])) {
                    pemEncryptionType = ((String)(JOptionPane.showInputDialog(((JFrame)(this.getParent())), "Please select the desired encryption type (default = AES-256-CFB)", "Select encryption format", JOptionPane.QUESTION_MESSAGE, null, pemEncryptionTypes, pemEncryptionTypes[9])));
                    if (pemEncryptionType == null) {return;}
                    if (pemEncryptionType.equals(pemEncryptionTypes[36]) == false) {
                        passphrase = KeyStoreHelper.enterPassphrase(((JFrame)(this.getParent())));
                    } else {
                        passphrase = "".toCharArray();
                    }
                } else {
                    passphrase = KeyStoreHelper.enterPassphrase(((JFrame)(this.getParent())));
                }
                if (passphrase == null) {return;}

                String fileName = KeyStoreHelper.saveDialog(((JFrame)(this.getParent())), null);
                if (fileName == null) {return;}

                       if (keyStoreType.equals(keyStoreTypes[0])) {
                    Object result = KeyStoreHelper.saveAsPFXNewType(keyStore, keyStorePassphrase, fileName, passphrase);
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                } else if (keyStoreType.equals(keyStoreTypes[1])) {
                    Object result = KeyStoreHelper.saveAsJKS(keyStore, keyStorePassphrase, fileName, passphrase);
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                } else if (keyStoreType.equals(keyStoreTypes[2])) {
                    Object result = KeyStoreHelper.saveAsBKS(keyStore, keyStorePassphrase, fileName, passphrase);
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                } else if (keyStoreType.equals(keyStoreTypes[3])) {
                    Object result = KeyStoreHelper.saveAsUBER(keyStore, keyStorePassphrase, fileName, passphrase);
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                } else if (keyStoreType.equals(keyStoreTypes[4])) {
                    Object result = KeyStoreHelper.saveAsJCEKS(keyStore, keyStorePassphrase, fileName, passphrase);
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                } else if (keyStoreType.equals(keyStoreTypes[5])) {
                    Object result = KeyStoreHelper.saveAsPEM(pemEncryptionType, keyStore, keyStorePassphrase, fileName, passphrase, new SecureRandom());
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                } else if (keyStoreType.equals(keyStoreTypes[6])) {
                    Object result = KeyStoreHelper.saveAsPFXOldType(keyStore, keyStorePassphrase, fileName, passphrase);
                    if (result != null) {
                        JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate with private key was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                    }

                }
            }

        } else if (exportPrivateKey == false) {
            try {
                keyStore.setCertificateEntry(KeyStoreHelper.getAlias(this.fChain[0]), this.fChain[0]);
            } catch (Exception e) {}

            String[] certificateTypes = new String[3];
            certificateTypes[0] = "DER encoded (.cer/.der)";
            certificateTypes[1] = "PEM encoded (.crt/.pem)";
            certificateTypes[2] = "As key store....";
            String certificateType = ((String)(JOptionPane.showInputDialog(((JFrame)(this.getParent())), "Please select the desired keystore type", "Select keystore format", JOptionPane.QUESTION_MESSAGE, null, certificateTypes, certificateTypes[1])));
            if (certificateType != null) {
                if (certificateType.equals(certificateTypes[2]) == true) {
                    String[] keyStoreTypes = new String[7];
                    keyStoreTypes[0] = "PKCS#12 keystore new type (.pfx/.p12)";
                    keyStoreTypes[6] = "PKCS#12 keystore old type (.pfx/.p12)";
                    keyStoreTypes[1] = "Java Key Store (.jks/.ts)";
                    keyStoreTypes[2] = "BouncyCastle.org Key Store (.bks/.bts)";
                    keyStoreTypes[3] = "UBER (3DES/TwoFish) formatted keystore (.uber)";
                    keyStoreTypes[4] = "JCEKS formatted keystore (.jceks)";
                    keyStoreTypes[5] = "OpenSSL PEM formatted (.pem)";

                    String[] pemEncryptionTypes = new String[37];
                    pemEncryptionTypes[ 0] = "AES-128-CBC";
                    pemEncryptionTypes[ 1] = "AES-128-CFB";
                    pemEncryptionTypes[ 2] = "AES-128-ECB";
                    pemEncryptionTypes[ 3] = "AES-128-OFB";
                    pemEncryptionTypes[ 4] = "AES-192-CBC";
                    pemEncryptionTypes[ 5] = "AES-192-CFB";
                    pemEncryptionTypes[ 6] = "AES-192-ECB";
                    pemEncryptionTypes[ 7] = "AES-192-OFB";
                    pemEncryptionTypes[ 8] = "AES-256-CBC";
                    pemEncryptionTypes[ 9] = "AES-256-CFB";
                    pemEncryptionTypes[10] = "AES-256-ECB";
                    pemEncryptionTypes[11] = "AES-256-OFB";
                    pemEncryptionTypes[12] = "BF-CBC";
                    pemEncryptionTypes[13] = "BF-CFB";
                    pemEncryptionTypes[14] = "BF-ECB";
                    pemEncryptionTypes[15] = "BF-OFB";
                    pemEncryptionTypes[16] = "DES-CBC";
                    pemEncryptionTypes[17] = "DES-CFB";
                    pemEncryptionTypes[18] = "DES-ECB";
                    pemEncryptionTypes[19] = "DES-OFB";
                    pemEncryptionTypes[20] = "DES-EDE";
                    pemEncryptionTypes[21] = "DES-EDE-CBC";
                    pemEncryptionTypes[22] = "DES-EDE-CFB";
                    pemEncryptionTypes[23] = "DES-EDE-ECB";
                    pemEncryptionTypes[24] = "DES-EDE-OFB";
                    pemEncryptionTypes[25] = "DES-EDE3";
                    pemEncryptionTypes[26] = "DES-EDE3-CBC";
                    pemEncryptionTypes[27] = "DES-EDE3-CFB";
                    pemEncryptionTypes[28] = "DES-EDE3-ECB";
                    pemEncryptionTypes[29] = "DES-EDE3-OFB";
                    pemEncryptionTypes[30] = "RC2-CBC";
                    pemEncryptionTypes[31] = "RC2-CFB";
                    pemEncryptionTypes[32] = "RC2-ECB";
                    pemEncryptionTypes[33] = "RC2-OFB";
                    pemEncryptionTypes[34] = "RC2-40-CBC";
                    pemEncryptionTypes[35] = "RC2-64-CBC";
                    pemEncryptionTypes[36] = "NO_ENCRYPTION";

                    String keyStoreType = ((String)(JOptionPane.showInputDialog(((JFrame)(this.getParent())), "Please select the desired keystore type", "Select keystore format", JOptionPane.QUESTION_MESSAGE, null, keyStoreTypes, keyStoreTypes[5])));
                    if (keyStoreType != null) {
                        String pemEncryptionType = null;
                        char[] passphrase = null;
                        if (keyStoreType.equals(keyStoreTypes[5])) {
                            pemEncryptionType = ((String)(JOptionPane.showInputDialog(((JFrame)(this.getParent())), "Please select the desired encryption type (default = AES-256-CFB)", "Select encryption format", JOptionPane.QUESTION_MESSAGE, null, pemEncryptionTypes, pemEncryptionTypes[9])));
                            if (pemEncryptionType == null) {return;}
                            if (pemEncryptionType.equals(pemEncryptionTypes[36]) == false) {
                                passphrase = KeyStoreHelper.enterPassphrase(((JFrame)(this.getParent())));
                            } else {
                                passphrase = "".toCharArray();
                            }
                        } else {
                            passphrase = KeyStoreHelper.enterPassphrase(((JFrame)(this.getParent())));
                        }
                        if (passphrase == null) {return;}

                        String fileName = KeyStoreHelper.saveDialog(((JFrame)(this.getParent())), null);
                        if (fileName == null) {return;}

                               if (keyStoreType.equals(keyStoreTypes[0])) {
                            Object result = KeyStoreHelper.saveAsPFXNewType(keyStore, keyStorePassphrase, fileName, passphrase);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }

                        } else if (keyStoreType.equals(keyStoreTypes[1])) {
                            Object result = KeyStoreHelper.saveAsJKS(keyStore, keyStorePassphrase, fileName, passphrase);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }

                        } else if (keyStoreType.equals(keyStoreTypes[2])) {
                            Object result = KeyStoreHelper.saveAsBKS(keyStore, keyStorePassphrase, fileName, passphrase);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }

                        } else if (keyStoreType.equals(keyStoreTypes[3])) {
                            Object result = KeyStoreHelper.saveAsUBER(keyStore, keyStorePassphrase, fileName, passphrase);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }

                        } else if (keyStoreType.equals(keyStoreTypes[4])) {
                            Object result = KeyStoreHelper.saveAsJCEKS(keyStore, keyStorePassphrase, fileName, passphrase);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }

                        } else if (keyStoreType.equals(keyStoreTypes[5])) {
                            Object result = KeyStoreHelper.saveAsPEM(pemEncryptionType, keyStore, keyStorePassphrase, fileName, passphrase, new SecureRandom());
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }

                        } else if (keyStoreType.equals(keyStoreTypes[6])) {
                            Object result = KeyStoreHelper.saveAsPFXOldType(keyStore, keyStorePassphrase, fileName, passphrase);

                        }
                    }
                } else if (certificateType.equals(certificateTypes[2]) == false) {
                    String fileName = KeyStoreHelper.saveDialog(((JFrame)(this.getParent())), null);
                    if (fileName != null) {
                        if (certificateType.equals(certificateTypes[0])) {
                            Object result = KeyStoreHelper.saveAsCER(this.fChain[0], fileName);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }
                        } else if (certificateType.equals(certificateTypes[1])) {
                            Object result = KeyStoreHelper.saveAsCRT(this.fChain[0], fileName);
                            if (result != null) {
                                JOptionPane.showMessageDialog(((JFrame)(this.getParent())), "Certificate was exported", "Export complete", JOptionPane.INFORMATION_MESSAGE);
                            }
                        }
                    }
                }
            }
        }
    }//GEN-LAST:event_btnSaveAsActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnOK;
    private javax.swing.JButton btnSaveAs;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTree x509PropertiesTree;
    // End of variables declaration//GEN-END:variables

}
