/* ============================================================================
 * X509SelectionDialog.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 *
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 *
 * ============================================================================
 *
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 *
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.keys;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyStore;
import java.security.KeyStore.PrivateKeyEntry;
import java.security.KeyStore.SecretKeyEntry;
import java.security.KeyStore.TrustedCertificateEntry;
import java.security.MessageDigest;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import java.security.spec.RSAPublicKeySpec;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import nl.jhhcvandermeijs.tcpconns.keys.KeyStoreHelper.DNProcessor;
import nl.jhhcvandermeijs.tcpconns.pgp.PGPHelper;
import nl.jhhcvandermeijs.tcpconns.utils.Utils;

/**
 *
 * @author username
 */
public class X509SelectionDialog extends javax.swing.JDialog {
    public static final long serialVersionUID = 1L;

    public static class Entry {
        String alias = null;
        String displayName = null;

        @Override()
        public String toString() {
            if ((this.displayName == null) || (this.displayName.equals(""))) {
                if ((alias == null) || (this.alias.equals(""))) {
                    return "n/a";
                } else {
                    return alias;
                }
            } else {
                return displayName;
            }
        }
    }

    KeyStore fKeyStore = null;
    char[] fPassphrase = null;

    public static X509Certificate[] showDialog(javax.swing.JFrame parent, List<X509Certificate> certsToChooseFrom) {
        X509Certificate[] ctcf = new X509Certificate[0];
        ctcf = certsToChooseFrom.toArray(ctcf);
        return X509SelectionDialog.showDialog(parent, ctcf);
    }

    public static X509Certificate[] showDialog(javax.swing.JFrame parent, X509Certificate[] certsToChooseFrom) {
        KeyStore keyStore = KeyStoreHelper.newEmptyKeyStore();
        SecureRandom secureRandom = new SecureRandom();
        byte[] buffer = new byte[64];
        secureRandom.nextBytes(buffer);
        String string = new String(buffer);
        char[] passphrase = string.toCharArray();
        int i = 1-1; int n = certsToChooseFrom.length;
        try {
            while (i <= (n-1)) {
                keyStore.setCertificateEntry(KeyStoreHelper.getAlias(certsToChooseFrom[i]), certsToChooseFrom[i]);
                i = i + 1;
            }
        } catch (Exception e) {}
        return X509SelectionDialog.showDialog(parent, keyStore, passphrase);
    }
    
    public static X509Certificate[] showDialog(javax.swing.JFrame parent, KeyStore keyStore, char[] passphrase) {
        X509SelectionDialog x509SelectionDialog = new X509SelectionDialog(parent, true);
        x509SelectionDialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        x509SelectionDialog.setLocationRelativeTo(parent);
        x509SelectionDialog.fKeyStore = keyStore;
        x509SelectionDialog.fPassphrase = passphrase;

        DefaultMutableTreeNode root = new DefaultMutableTreeNode(); root.setUserObject("Available Certificates");
        try {
            Enumeration<String> aliases = keyStore.aliases();
            String alias = null;
            while (aliases.hasMoreElements()) {
                alias = aliases.nextElement();
                if (keyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                    try {
                        PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(keyStore.getEntry(alias, new KeyStore.PasswordProtection(passphrase))));
                        if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                            X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                            Entry entry = new Entry();
                            entry.alias = alias;
                            entry.displayName = (new DNProcessor(chain[0].getSubjectDN().getName())).getCommonName();
                            DefaultMutableTreeNode node = new DefaultMutableTreeNode();
                            node.setUserObject(entry);
                            root.add(node);
                        }                        
                    } catch (Exception e2) {}

                } else if (keyStore.entryInstanceOf(alias, SecretKeyEntry.class)) {
                    try {
                        //for now do nothing
                    } catch (Exception e2) {}

                } else if (keyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                    try {
                        TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(keyStore.getEntry(alias, null)));
                        if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                            X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                            Entry entry = new Entry();
                            entry.alias = alias;
                            entry.displayName = (new DNProcessor(x509.getSubjectDN().getName())).getCommonName();
                            DefaultMutableTreeNode node = new DefaultMutableTreeNode();
                            node.setUserObject(entry);
                            root.add(node);
                        }
                    } catch (Exception e2) {}
                }
            }
        } catch (Exception e) {
            System.out.println(e);        
        }
        DefaultTreeModel defaultTreeModel = new DefaultTreeModel(root);
        x509SelectionDialog.keyStoreTree.setModel(defaultTreeModel);
        x509SelectionDialog.x509PropertiesTree.setModel(null);


        x509SelectionDialog.setVisible(true);


        try {
            X509Certificate[] selectedCertificates = null;
            List<X509Certificate> list = new LinkedList<X509Certificate>();
            if (x509SelectionDialog.keyStoreTree.getModel() == null) {
                //do nothing
            } else {
                if (x509SelectionDialog.keyStoreTree.getSelectionCount() == 0) {
                    selectedCertificates = new X509Certificate[0];
                } else {
                    TreePath[] treePaths = x509SelectionDialog.keyStoreTree.getSelectionPaths();
                    if (treePaths != null) {
                        TreePath treePath = null;
                        int i = 1-1; int n = treePaths.length;
                        while (i <= (n-1)) {
                            treePath = treePaths[i];
                            Object[] objects = treePath.getPath();
                            Object object = objects[objects.length-1];
                            if (object instanceof DefaultMutableTreeNode) {
                                DefaultMutableTreeNode node = ((DefaultMutableTreeNode)(object));
                                if (node.getUserObject() instanceof Entry) {
                                    Entry entry = ((Entry)(node.getUserObject()));
                                    String alias = entry.alias;

                                    if (keyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                                        try {
                                            PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(keyStore.getEntry(alias, new KeyStore.PasswordProtection(passphrase))));
                                            if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                                                X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                                                list.add(chain[0]);
                                            }
                                        } catch (Exception e2) {}

                                    } else if (keyStore.entryInstanceOf(alias, SecretKeyEntry.class)) {
                                        try {
                                            //for now do nothing
                                        } catch (Exception e2) {}

                                    } else if (keyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                                        try {
                                            TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(keyStore.getEntry(alias, null)));
                                            if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                                                X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                                                list.add(x509);
                                            }
                                        } catch (Exception e2) {}
                                    }

                                }
                            }
                            i = i + 1;
                        }
                        selectedCertificates = new X509Certificate[0];
                        selectedCertificates = list.toArray(selectedCertificates);
                    }
                }
            }
            return selectedCertificates;
        } catch (Exception e) {
            System.out.println(e);
            return null;
        }
    }

    public void updateX509PropertiesTree(X509Certificate[] chain, PrivateKey privateKey, JTree jTree) {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode(); root.setUserObject("v" + chain[0].getVersion() + " X.509 " + chain[0].getPublicKey().getAlgorithm() + " (Serial no.: " + Utils.toHex(chain[0].getSerialNumber().toByteArray(), true) + ")");

        if (chain[0].getSigAlgName().equals("1.2.840.113549.1.1.10")) {
            DefaultMutableTreeNode signatureType = new DefaultMutableTreeNode(); signatureType.setUserObject("Signature Algorithm:  RSASSA_PSS");
                root.add(signatureType);
        } else {
            DefaultMutableTreeNode signatureType = new DefaultMutableTreeNode(); signatureType.setUserObject("Signature Algorithm:  " + chain[0].getSigAlgName());
                root.add(signatureType);
        }

        DefaultMutableTreeNode keySize = new DefaultMutableTreeNode();
            if (chain[0].getPublicKey().getAlgorithm().toUpperCase().contains("RSA")) {
                try {
                    KeyFactory keyFactory = KeyFactory.getInstance("RSA");
                    RSAPublicKeySpec rsaPublicKeySpec = ((RSAPublicKeySpec)(keyFactory.getKeySpec(chain[0].getPublicKey(), RSAPublicKeySpec.class)));
                    BigInteger bigInteger = rsaPublicKeySpec.getModulus();
                    keySize.setUserObject("Keysize (in bits): " + bigInteger.toString(2).length());
                } catch (Exception e) {
                    keySize.setUserObject("Keysize: unknown keysize");
                }
            }
            root.add(keySize);

        if (privateKey != null) {
            DefaultMutableTreeNode privkey = new DefaultMutableTreeNode(); privkey.setUserObject("Private Key:  present");
            root.add(privkey);
        } else {
            DefaultMutableTreeNode privkey = new DefaultMutableTreeNode(); privkey.setUserObject("Private Key:  n/a");
            root.add(privkey);
        }

        if (KeyStoreHelper.isSelfSigned(chain[0])) {
            DefaultMutableTreeNode selfsigned = new DefaultMutableTreeNode(); selfsigned.setUserObject("Self-signed:  true");
            root.add(selfsigned);
        } else {
            DefaultMutableTreeNode selfsigned = new DefaultMutableTreeNode(); selfsigned.setUserObject("Self-signed:  false");
            root.add(selfsigned);
        }

        int pathDepth = chain[0].getBasicConstraints();
        if (pathDepth == -1) {
            DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  false");
            root.add(isCA);
        } else {
            if (pathDepth == Integer.MAX_VALUE) {
                DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  true (path depth: unlimited)");
                root.add(isCA);
            } else {
                DefaultMutableTreeNode isCA = new DefaultMutableTreeNode(); isCA.setUserObject("Is or can act as CA:  true (path depth: " + pathDepth + ")");
                root.add(isCA);
            }
        }

        DNProcessor dn = new DNProcessor(chain[0]);
        this.lblX509Properties.setText("Properties of \"" + dn.getCommonName() + "\"");
        DefaultMutableTreeNode subject = new DefaultMutableTreeNode(); subject.setUserObject("Issued To");
            DefaultMutableTreeNode subjectCN = new DefaultMutableTreeNode(); subjectCN.setUserObject("Common Name:  " + dn.getCommonName()); subject.add(subjectCN);
            DefaultMutableTreeNode subjectOU = new DefaultMutableTreeNode(); subjectOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); subject.add(subjectOU);
            DefaultMutableTreeNode subjectO  = new DefaultMutableTreeNode(); subjectO.setUserObject("Organization:  " + dn.getOrganization()); subject.add(subjectO);
            DefaultMutableTreeNode subjectL  = new DefaultMutableTreeNode(); subjectL.setUserObject("Locality:  " + dn.getLocality()); subject.add(subjectL);
            DefaultMutableTreeNode subjectST = new DefaultMutableTreeNode(); subjectST.setUserObject("State or Province:  " + dn.getStateOrProvince()); subject.add(subjectST);
            DefaultMutableTreeNode subjectC  = new DefaultMutableTreeNode(); subjectC.setUserObject("Country:  " + dn.getCountry()); subject.add(subjectC);
            DefaultMutableTreeNode subjectUID = new DefaultMutableTreeNode(); subjectUID.setUserObject("Unique ID: " + dn.getUID()); subject.add(subjectUID);
            DefaultMutableTreeNode subjectEmail = new DefaultMutableTreeNode(); subjectEmail.setUserObject("Email: " + dn.getEmailAddress()); subject.add(subjectEmail);
            DefaultMutableTreeNode subjectWebsite = new DefaultMutableTreeNode(); subjectWebsite.setUserObject("Website: " + dn.getURI()); subject.add(subjectWebsite);
            DefaultMutableTreeNode subjectIP = new DefaultMutableTreeNode(); subjectIP.setUserObject("IPAddress: " + dn.getIPAddress()); subject.add(subjectIP);
            root.add(subject);

        dn = new DNProcessor(chain[0].getIssuerDN().getName());
        DefaultMutableTreeNode issuer = new DefaultMutableTreeNode(); issuer.setUserObject("Issued By");
            DefaultMutableTreeNode issuerCN = new DefaultMutableTreeNode(); issuerCN.setUserObject("Common Name:  " + dn.getCommonName()); issuer.add(issuerCN);
            DefaultMutableTreeNode issuerOU = new DefaultMutableTreeNode(); issuerOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); issuer.add(issuerOU);
            DefaultMutableTreeNode issuerO  = new DefaultMutableTreeNode(); issuerO.setUserObject("Organization:  " + dn.getOrganization()); issuer.add(issuerO);
            DefaultMutableTreeNode issuerL  = new DefaultMutableTreeNode(); issuerL.setUserObject("Locality:  " + dn.getLocality()); issuer.add(issuerL);
            DefaultMutableTreeNode issuerST = new DefaultMutableTreeNode(); issuerST.setUserObject("State or Province:  " + dn.getStateOrProvince()); issuer.add(issuerST);
            DefaultMutableTreeNode issuerC  = new DefaultMutableTreeNode(); issuerC.setUserObject("Country:  " + dn.getCountry()); issuer.add(issuerC);
            DefaultMutableTreeNode issuerUID = new DefaultMutableTreeNode(); issuerUID.setUserObject("Unique ID: " + dn.getUID()); issuer.add(issuerUID);
            root.add(issuer);

        if (chain.length > 1) {
            int i = 1-1+1; int n = chain.length-1;
            while (i <= (n-1)) {
                if (KeyStoreHelper.isSelfSigned(chain[i]) == false) {
                    dn = new DNProcessor(chain[i].getIssuerDN().getName());
                    issuer = new DefaultMutableTreeNode(); issuer.setUserObject("Issued By");
                        issuerCN = new DefaultMutableTreeNode(); issuerCN.setUserObject("Common Name:  " + dn.getCommonName()); issuer.add(issuerCN);
                        issuerOU = new DefaultMutableTreeNode(); issuerOU.setUserObject("Organizational Unit:  " + dn.getOrganizationalUnit()); issuer.add(issuerOU);
                        issuerO  = new DefaultMutableTreeNode(); issuerO.setUserObject("Organization:  " + dn.getOrganization()); issuer.add(issuerO);
                        issuerL  = new DefaultMutableTreeNode(); issuerL.setUserObject("Locality:  " + dn.getLocality()); issuer.add(issuerL);
                        issuerST = new DefaultMutableTreeNode(); issuerST.setUserObject("State or Province:  " + dn.getStateOrProvince()); issuer.add(issuerST);
                        issuerC  = new DefaultMutableTreeNode(); issuerC.setUserObject("Country:  " + dn.getCountry()); issuer.add(issuerC);
                        issuerUID = new DefaultMutableTreeNode(); issuerUID.setUserObject("Unique ID: " + dn.getUID()); issuer.add(issuerUID);
                        root.add(issuer);
                }
                i = i + 1;
            }
        }

        DefaultMutableTreeNode validityPeriod = new DefaultMutableTreeNode(); validityPeriod.setUserObject("Validity Period");
            DefaultMutableTreeNode notBefore = new DefaultMutableTreeNode(); notBefore.setUserObject("Not Before:  " + chain[0].getNotBefore()); validityPeriod.add(notBefore);
            DefaultMutableTreeNode notAfter = new DefaultMutableTreeNode(); notAfter.setUserObject("Not After:  " + chain[0].getNotAfter()); validityPeriod.add(notAfter);
            DefaultMutableTreeNode currentlyValid = new DefaultMutableTreeNode();
            try {
                chain[0].checkValidity();
                currentlyValid.setUserObject("Currently *within* validity period".toUpperCase());
            } catch (CertificateExpiredException e) {
                currentlyValid.setUserObject("N.B.(!): Certificate *is no longer valid*".toUpperCase());
                this.lblX509Properties.setText("N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" HAS EXPIRED!");
                //JOptionPane.showMessageDialog(null, "N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" has EXPIRED!", "Certificate Validity Warning", JOptionPane.WARNING_MESSAGE);
            } catch (CertificateNotYetValidException e2) {
                currentlyValid.setUserObject("N.B.(!): Certificate *not yet valid*".toUpperCase());
                this.lblX509Properties.setText("N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" IS NOT(!) YET VALID");
                //JOptionPane.showMessageDialog(null, "N.B.(!):  \"" + (new KeyStoreHelper.DNProcessor(chain[0].getSubjectDN().getName())).getCommonName() + "\" is NOT(!) YET VALID", "Certificate Validity Warning", JOptionPane.WARNING_MESSAGE);
            }                                
            validityPeriod.add(currentlyValid);
            root.add(validityPeriod);

        DefaultMutableTreeNode pubkey = new DefaultMutableTreeNode();
        pubkey.setUserObject("Public key:  " + chain[0].getPublicKey().toString());
        root.add(pubkey);

        DefaultMutableTreeNode signature = new DefaultMutableTreeNode();
        signature.setUserObject("Signature: " + Utils.toHex(chain[0].getSignature(), true));
        root.add(signature);

        try {
            boolean[] keyUsage = chain[0].getKeyUsage();
            if (keyUsage != null) {
                DefaultMutableTreeNode keyUsageNode = new DefaultMutableTreeNode();
                //keyUsageNode.setUserObject("Key Usage");
                keyUsageNode.setUserObject("Key Usage (" + Utils.toHex(chain[0].getExtensionValue("2.5.29.15"), true) + ")");
                if (keyUsage[0] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("digitalSignature: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("digitalSignature: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[1] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("nonRepudiation: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("nonRepudiation: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[2] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyEncipherment: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyEncipherment: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[3] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("dataEncipherment: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("dataEncipherment: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[4] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyAgreement: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyAgreement: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[5] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyCertSign: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("keyCertSign: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[6] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("cRLSign: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("cRLSign: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[7] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("encipherOnly: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("encipherOnly: NO");
                    keyUsageNode.add(ku);
                }
                if (keyUsage[8] == true) {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("decipherOnly: YES");
                    keyUsageNode.add(ku);
                } else {
                    DefaultMutableTreeNode ku = new DefaultMutableTreeNode();
                    ku.setUserObject("decipherOnly: NO");
                    keyUsageNode.add(ku);
                }
                root.add(keyUsageNode);
            }
        } catch (Exception e) {}

        DefaultMutableTreeNode fingerprints = new DefaultMutableTreeNode(); fingerprints.setUserObject("Fingerprints");
            KeyStoreHelper.assertProviders();
            try {
                byte[] encodedCert = chain[0].getEncoded();
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-512", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-512:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-256", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-256:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA1");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA1:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD5");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD5:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("Whirlpool", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("Whirlpool:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("GOST3411", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("GOST3411:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD128", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD128:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD160", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD160:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD256", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD256:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("RipeMD320", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("RipeMD320:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-224", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-224:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("SHA-384", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("SHA-384:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("Tiger", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("Tiger:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD4", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD4:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
                try {
                    MessageDigest messageDigest = MessageDigest.getInstance("MD2", "BC");
                    messageDigest.update(encodedCert);
                    DefaultMutableTreeNode hash = new DefaultMutableTreeNode("MD2:  " + Utils.toHex(messageDigest.digest(), true));
                    fingerprints.add(hash);
                } catch (Exception e) {}
            root.add(fingerprints);
            } catch (Exception e2) {}

        try {
        DefaultMutableTreeNode pgpData = new DefaultMutableTreeNode(); pgpData.setUserObject("PGP");
        DefaultMutableTreeNode pgpKeyID = new DefaultMutableTreeNode(); pgpKeyID.setUserObject("PGP KeyID:  " + PGPHelper.getPGPKeyID(chain[0])); pgpData.add(pgpKeyID);
        DefaultMutableTreeNode pgpFingerprint = new DefaultMutableTreeNode(); pgpFingerprint.setUserObject("PGP Fingerprint:  " + PGPHelper.getPGPFingerprintAsHexEncodedString(chain[0], true)); pgpData.add(pgpFingerprint);
            root.add(pgpData);
        } catch (Exception e) {}

        DefaultTreeModel defaultTreeModel = new DefaultTreeModel(root);
        jTree.setModel(defaultTreeModel);

        int i = 1-1; int n = root.getChildCount();
        while (i <= (n-1)) {
            jTree.expandPath(new TreePath(defaultTreeModel.getPathToRoot(root.getChildAt(i))));
            int j = 1-1; int m = root.getChildAt(i).getChildCount();
            while (j <= (m-1)) {
                jTree.expandPath(new TreePath(defaultTreeModel.getPathToRoot(root.getChildAt(i).getChildAt(j))));
                j = j + 1;
            }
            i = i + 1;
        }
    }

    /** Creates new form X509PropertiesDialog */
    public X509SelectionDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnOK = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        x509PropertiesTree = new javax.swing.JTree();
        jScrollPane2 = new javax.swing.JScrollPane();
        keyStoreTree = new javax.swing.JTree();
        lblX509Properties = new javax.swing.JLabel();
        btnCancel = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Please select the appropriate certificate(s)...");

        btnOK.setText("  OK  ");
        btnOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOKActionPerformed(evt);
            }
        });

        jScrollPane1.setViewportView(x509PropertiesTree);

        keyStoreTree.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                keyStoreTreeValueChanged(evt);
            }
        });
        jScrollPane2.setViewportView(keyStoreTree);

        lblX509Properties.setText("Certificate Properties:");

        btnCancel.setText("Cancel");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 169, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(6, 6, 6)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(lblX509Properties)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(btnCancel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btnOK))
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 431, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(15, 15, 15)
                .add(lblX509Properties)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 362, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 362, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnOK)
                    .add(btnCancel))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOKActionPerformed
        this.dispose();
    }//GEN-LAST:event_btnOKActionPerformed

    private void keyStoreTreeValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_keyStoreTreeValueChanged
        TreePath treePath = this.keyStoreTree.getSelectionPath();
        if (treePath == null) {
            this.x509PropertiesTree.setModel(null);
            this.lblX509Properties.setText("No certificate selected. ");
        } else {
            if ((this.keyStoreTree.getSelectionCount() == 0) || (this.keyStoreTree.getSelectionCount() > 1)) {
                this.x509PropertiesTree.setModel(null);
                if (this.keyStoreTree.getSelectionCount() > 1) {
                    this.lblX509Properties.setText("Multiple certificates selected. ");
                } else {
                    this.lblX509Properties.setText("No certificate selected. ");
                }
            } else {
                Object[] objects = treePath.getPath();
                Object object = objects[objects.length-1];
                if ((object instanceof DefaultMutableTreeNode) == false) { 
                    this.x509PropertiesTree.setModel(null);
                    this.lblX509Properties.setText("No certificate selected. ");
                } else {
                    DefaultMutableTreeNode node = ((DefaultMutableTreeNode)(object));
                    if (node.getUserObject() instanceof Entry) {
                        Entry entry = ((Entry)(node.getUserObject()));
                        String alias = entry.alias;
                        try {
                            if (this.fKeyStore.entryInstanceOf(alias, PrivateKeyEntry.class)) {
                                try {
                                    PrivateKeyEntry privateKeyEntry = ((PrivateKeyEntry)(this.fKeyStore.getEntry(alias, new KeyStore.PasswordProtection(this.fPassphrase))));
                                    if (privateKeyEntry.getCertificateChain() instanceof X509Certificate[]) {
                                        X509Certificate[] chain = ((X509Certificate[])(privateKeyEntry.getCertificateChain()));
                                        PrivateKey privateKey = privateKeyEntry.getPrivateKey();
                                        this.updateX509PropertiesTree(chain, privateKey, this.x509PropertiesTree);
                                    }
                                } catch (Exception e2) {}

                            } else if (this.fKeyStore.entryInstanceOf(alias, SecretKeyEntry.class)) {
                                try {
                                    //for now do nothing
                                } catch (Exception e2) {}

                            } else if (this.fKeyStore.entryInstanceOf(alias, TrustedCertificateEntry.class)) {
                                try {
                                    TrustedCertificateEntry trustedCertificateEntry = ((TrustedCertificateEntry)(this.fKeyStore.getEntry(alias, null)));
                                    if (trustedCertificateEntry.getTrustedCertificate() instanceof X509Certificate) {
                                        X509Certificate x509 = ((X509Certificate)(trustedCertificateEntry.getTrustedCertificate()));
                                        X509Certificate[] chain = new X509Certificate[1];
                                        chain[0] = x509;
                                        this.updateX509PropertiesTree(chain, null, this.x509PropertiesTree);
                                    }
                                } catch (Exception e2) {}
                            }
                        } catch (Exception e) {
                            JOptionPane.showMessageDialog(null, "There was an error \"" + e + "\"", "Something did no go right", JOptionPane.ERROR_MESSAGE);
                        }
                    } else {
                        this.x509PropertiesTree.setModel(null);
                        this.lblX509Properties.setText("No certificate selected. ");
                    }
                }
            }
        }
    }//GEN-LAST:event_keyStoreTreeValueChanged

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        this.x509PropertiesTree.setModel(null);
        this.keyStoreTree.setModel(null);
        this.dispose();
    }//GEN-LAST:event_btnCancelActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnOK;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTree keyStoreTree;
    private javax.swing.JLabel lblX509Properties;
    private javax.swing.JTree x509PropertiesTree;
    // End of variables declaration//GEN-END:variables

}
