/* ============================================================================
 * TTempFile.java
 * ============================================================================
 * THIS SOFTWARE IS RELEASED UNDER THE FOLLOWING BINDING LEGAL AGREEMENT
 * (BSD-LICENSE).
 * ============================================================================
 *
 * Copyright (C) 2007-2009
 * by J.H.H.C. van der Meijs (tcpconns@jhhcvandermeijs.nl)
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice, 
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of tcpconns nor the names of its copyright holders or 
 *      its contributors may be used to endorse or promote products derived from 
 *      this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * ============================================================================
 * IF YOU DO NOT AGREE TO THE ABOVE TERMS AND CONDITIONS THEN DO *NOT* USE THIS 
 * SOFTWARE AND *DO* DESTROY ALL COPIES OF THIS SOFTWARE THAT YOU HAVE.
 * ============================================================================
 * 
 * TCPCONNS’ CURRENT CODE IS NOT KNOWN TO VIOLATE ANY COPYRIGHTS OR INFRINGE ON 
 * ANY PATENTS. MOREOVER, IT ADHERES TO GENERAL PROGRAMMING RULES AND PRACTICES 
 * AND IS OF SUCH A SIMPLISTIC NATURE THAT A SIMILAR PACKAGE COULD BE 
 * CONSTRUCTED BY ANYONE. IMPORTANT(!): IF YOU FEEL THAT A PIECE OF CODE IN
 * TCPCONNS VIOLATES A COPYRIGHT OR INFRINGES ON A PATENT, PLEASE DO INFORM ONE 
 * OF THE AUTHORS IMMEDIATELY. DO ALWAYS STATE EXACTLY WHAT CODE IS NOT ALLOWED
 * UNDER EXACTLY WHAT RULING, COPYRIGHT, OR PATENT; AND PLEASE DO ELABORATE ON 
 * EXACTLY HOW THE CODE VIOLATES THE SPECIFIC COPYRIGHT OR INFRINGES ON
 * THE SPECIFIC PATENT.
 * 
 * ============================================================================
 * 
 * This software is Open Source Initiative approved Open Source Software.
 * Open Source Initiative Approved is a trademark of the Open Source Initiative.
 * http://www.opensource.org/
 * 
 * ============================================================================
 */
package nl.jhhcvandermeijs.tcpconns.utils;

import java.io.File;
import java.security.SecureRandom;

/**
 * use this class to obtain a temporary file handle.
 */
public class TTempFile {
    /**
     * This random number generator is utilized by the newTempFile method
     * to further increase the likelihood that the generated filename indeed
     * is unique.
     */
    protected static SecureRandom SRNG = new SecureRandom();
    
    /**
     * Use this method to obtain a temporary file. 
     * 
     * @param applicationName a String that you can use to identify your
     *                        application. This element further increases
     *                        the chance of name uniqueness.
     * @param extension a String that should hold the extension the tempfile
     *        should have; if null then the extension will be .tmp
     * @param deleteOnExit if true the tempfile will be deleted when the
     *         process exits gracefully; preferentially you should delete a 
     *         tempfile yourself immediately after the useful period of the
     *         tempfile has expired
     * @return returns a File with an unique filename (applicationName that you
     *         choose, 2 x a random number, the current thread id, and the
     *         currenttime in milli seconds + '.tmp')
     */
    synchronized static public File newTempFile(String applicationName, String extension, boolean deleteOnExit) {
            if (applicationName == null) {applicationName = "tcpconns";}
            if (extension == null) {extension = ".tmp";}

            File tempFile = null; String tempFileName = null;
            try {
                 tempFileName = applicationName
                                + Math.abs(TTempFile.SRNG.nextLong())
                                + Math.abs(TTempFile.SRNG.nextLong()) 
                                + Thread.currentThread().getId()                                 
                                + System.currentTimeMillis();
                 tempFile = File.createTempFile(tempFileName, extension);
                 tempFile.createNewFile();
                 tempFileName = tempFile.getCanonicalPath();
            } catch (Exception e) {                     
                try {tempFile.delete();} catch (Exception e2) {}
                return null;
            }            
            if (tempFile.exists()) {
                try {tempFile.delete();} catch (Exception e2) {}
            }                            
            tempFile = new File(tempFileName);
            if (deleteOnExit) {
                tempFile.deleteOnExit();
            }
            return tempFile;
        }

    /**
     * Use this method to obtain a temporary file handle
     *
     */
    synchronized static public File newTempFile() {
        return TTempFile.newTempFile(null, null, true);
    }

}
